#==========================================================+==============================
# package      DE_EPAGES::Catalog::API::Attributes::Category
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Attribute interface for Product.
#========================================================================================
package DE_EPAGES::Catalog::API::Attributes::Category;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::Catalog::API::Catalog qw ( AssignProductToCategory RenumberCategoryProducts );
use DE_EPAGES::Core::API::PerlTools qw ( unique );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadClassByAlias );
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );

#========================================================================================
# function     getAttributes
# state        private
# syntax       $Value = $Object->get($AttributeName);
#========================================================================================
sub getAttributes {
    shift;
    my ($Object, $aNames) = @_;
    my $hInfo = {};
    my $CategoryProductMapClass = LoadClassByAlias('CategoryProductMap');
    foreach my $Name (@$aNames) {
        if ($Name eq 'VisibleProducts') {
            $hInfo->{$Name} = _VisibleProducts($Object);
        }
        elsif ($Name eq 'Products') {
            $hInfo->{$Name} = [map { $_->get('Product') }
                               sort {$a->get('Position') <=> $b->get('Position')}
                               @{ $CategoryProductMapClass->getObjects('ByCategoryID', [$Object->id]) }
                              ];
        }
        elsif ($Name eq 'ProductsThreeColumnTemplate') {
            my $PageType = LoadPageTypeByAlias('SF-Category');
            my $TemplateTypeAlias = $PageType->templateTypeAlias( $Object, 'Content-Products' );
            $hInfo->{$Name} = $TemplateTypeAlias eq 'ProductsThreeColumns' ? 1 : 0;
        }
        elsif ($Name eq 'ProductCount') {
            $hInfo->{$Name} = scalar @{ $CategoryProductMapClass->findObjects('ByCategoryID', [$Object->id]) };
        }
        elsif ($Name eq 'CategoryProductMaps') {
            $hInfo->{$Name} = [sort {$a->get('Position') <=> $b->get('Position')}
                               @{ $CategoryProductMapClass->getObjects('ByCategoryID', [$Object->id]) }];
        }
        elsif ($Name eq 'IsSpecialOffer') {
            $hInfo->{$Name} = $Object->instanceOf('CategorySpecialOffer') ? 1 : 0;
        }
    }
    return $hInfo;
}

#========================================================================================
# function     deleteObject
# state        private
# description  remove the categoryproductmap objects when a category is being deleted
#========================================================================================
sub deleteObject {
    shift;
    my ($Category) = @_;

    $_->delete foreach @{$Category->get('CategoryProductMaps')};
}

#========================================================================================
# function     cloneAttributes
# state        private
# description  clone assigned attributes of object
#========================================================================================
sub cloneAttributes {
    shift;
    my ($Object, $Clone) = @_;
    # clone assigned products
    my $aProductMaps = $Object->get('CategoryProductMaps');
    foreach my $ProductMap(@$aProductMaps) {
        my $Product = $ProductMap->get('Product');
        my $Position = $ProductMap->get('Position');
        AssignProductToCategory($Product, $Clone, $Position);
    }
    RenumberCategoryProducts($Clone);
}

#========================================================================================
# function     exportXMLElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->exportXMLElements($Driver, $Object, $aNames, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Adds sub elements of object for xml export.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver | object
# input        $Object | object to export | object
# input        $aNames | attribute names | ref.array.string
# input        $hAttributes | imported attributes | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $Object, $aNames, $hAttributes) = @_;
    my $CategoryProductMapClass = LoadClassByAlias('CategoryProductMap');
    if ($CategoryProductMapClass->get('ExportLevel') == $Driver->exportLevel) {
        foreach my $CategoryProductMap ( @{ $Object->get('CategoryProductMaps') } ) {
            $Driver->addObject( 'CategoryProductMap', $CategoryProductMap );
        }
    }
    return;
}

#========================================================================================
# function     _VisibleProducts
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aProducts = _VisibleProducts($Category);
#----------------------------------------------------------------------------------------
# description  Returns all visible products of a category.
#               Special handling for variation products:
#               If the category contains a sub products, the product is included if
#               the sub product is visible, even if the master product is not.
#               If the category contains a master product and the master product is
#               visible, then the default sub product product is used.
#               If the default sub product product is not visibles, then any visible
#               sub product is used.
#               If no sub product is ivisible, then the product is not included.
#----------------------------------------------------------------------------------------
# input        $Category | category | object
# return       $aProducts | list of products | ref.array.object
#========================================================================================
sub _VisibleProducts {
    my ($Category) = @_;
    my @ProductIDs = map { $_->{'ProductID'} } sort { $a->{'Position'} <=> $b->{'Position'} } @{LoadClassByAlias('CategoryProductMap')->dalObject->get('ByCategoryID', [$Category->id])};
    return [] unless scalar @ProductIDs;

    my $dal = LoadClassByAlias('Product')->packageObject->dal;
    my $hProducts = $dal->multi(\@ProductIDs);

    # preload default sub products
    my @DefaultProductIDs = unique map { $_->{'DefaultSubProductID'} } grep { defined $_->{'DefaultSubProductID'}} @$hProducts;
    $dal->multi(\@DefaultProductIDs) if scalar @DefaultProductIDs;

    my (@Products, %Unique);
    foreach my $ProductID (@ProductIDs) {
        my $Product = LoadObject($ProductID);
        next unless $Product->get('IsVisible');

        my $SubProduct = $Product->get('DefaultSubProduct');
        if (defined $SubProduct) {
            ($Product) = $SubProduct->get('IsVisible') ? $SubProduct : grep { $_->get('IsVisible') } @{$Product->get('SubProducts')};
        }
        if (defined $Product and $Product->get('IsVisible') and not exists $Unique{$Product->id}) {
            push @Products, $Product;
            $Unique{$Product->id} = undef;
        }
    }
    return \@Products;
}

1;
