#========================================================================================
# §package      DE_EPAGES::Catalog::DAL::mysql::CategoryProductMap
# §state        private
# §description  Generic mysql database interface for tables:
#               <ul>
#                   <li>categoryproductmap
#               </ul>
#========================================================================================
package DE_EPAGES::Catalog::DAL::mysql::CategoryProductMap;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( GetLoopFromDoubleArray );

#========================================================================================
# §function     GetProductIDsByCategoryID
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       DE_EPAGES::Catalog::DAL::mysql::CategoryProductMap->GetProductIDsByCategoryID($dbi, $CategoryID);
#----------------------------------------------------------------------------------------
# §description  get affected categoryproductmapids for categoryid
#----------------------------------------------------------------------------------------
# §input        $CategoryID | category id | integer
#========================================================================================
sub GetProductIDsByCategoryID{
    my $self = shift;
    my ($dbi, $CategoryID) = @_;

    my $sql = 'SELECT categoryproductmapid FROM categoryproductmap cpm WHERE cpm.categoryid=?';
    my $dbResult = $dbi->execute($sql, [$CategoryID], ['Integer']);
    return $dbResult;
}

#========================================================================================
# §function     GetProductTreeNodeAttributesByCategoryID
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       DE_EPAGES::Catalog::DAL::mysql::CategoryProductMap->GetProductTreeNodeAttributesByCategoryID($dbi, $CategoryID, $LanguageID);
#----------------------------------------------------------------------------------------
# §description  get attributes of products by categoryid that are needed to construct
#               tree at Content/Categories content preview page. LanguageId is language
#               currently used by the user. Returns a table with columns:
#               | alias | name | objectid | classid | classalias | position | isvisible | ismaster | quality |
#----------------------------------------------------------------------------------------
# §input        $CategoryID | category id | integer
# §input        $LanguageID | language id | integer
# §return       $ahDBResult   | array of hashes, where each index in array represents a row and in each hash column names serve as keys.
#               following keys are used:
#               <ul>
#                   <li>alias</li>
#                   <li>name/li>
#                   <li>objectid</li>
#                   <li>classid</li>
#                   <li>classalias</li>
#                   <li>position</li>
#                   <li>isvisible</li>
#                   <li>ismaster</li>
#                   <li>quality</li>
#               </ul> | reference to array of hashes
#========================================================================================
sub GetProductTreeNodeAttributesByCategoryID{
    my $self = shift;
    my ($dbi, $CategoryID, $LanguageID) = @_;

    my $sql = q(
SELECT
    o.alias,
    lsa.value AS name,
    o.objectid,
    o.classid,
    class.alias AS classalias,
    o.position,
    co.isvisible,
    p.ismaster,
    sq.quality
FROM
    object AS o
        INNER JOIN
    categoryproductmap AS cpm ON cpm.productid = o.objectid
        INNER JOIN
    contentobject AS co ON o.objectid = co.contentobjectid
        INNER JOIN
    product AS p ON o.objectid = p.productid
        LEFT JOIN
    object AS class ON o.classid = class.objectid
        LEFT JOIN
    seoquality AS sq ON o.objectid = sq.contentobjectid
        AND sq.languageid = ?
        LEFT JOIN
    localizedstringattribute AS lsa ON o.objectid = lsa.attributeid
        AND lsa.languageid = ?
        AND o.alias = 'Name'
WHERE
    cpm.categoryid = ?;);

    my $ahDBResult = $dbi->execute($sql, [$LanguageID, $LanguageID, $CategoryID], ['Integer', 'Integer', 'Integer']);
    return GetLoopFromDoubleArray($ahDBResult, [qw ( alias name objectid classid classalias position isvisible ismaster quality )]);
}

#========================================================================================
# §function     RenumberCategoryProductsByCategoryID
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       DE_EPAGES::Catalog::DAL::mysql::CategoryProductMap->RenumberCategoryProductsByCategoryID($dbi, $CategoryID, $StepSize);
#----------------------------------------------------------------------------------------
# §description  renumbers all categoryproductmaps position for a specific category id
#               renumber is done on database for high performance
#----------------------------------------------------------------------------------------
# §input        $CategoryID | category id | integer
# §input        $StepSize | step size e.g. 10 | integer
#========================================================================================
sub RenumberCategoryProductsByCategoryID{
    my $self = shift;
    my ($dbi, $CategoryID, $StepSize) = @_;

    my $sql_set_variable = 'SET @row_number=0';
    my $sql_renumber = 'UPDATE categoryproductmap ' .
        'SET position=(@row_number:=@row_number+?) ' .
        'WHERE categoryid=? ' .
        'ORDER BY position ASC';

    $dbi->do($sql_set_variable);
    $dbi->do($sql_renumber, [$StepSize, $CategoryID]);

    return 1;
}

1;
