#========================================================================================
# package      DE_EPAGES::Catalog::Hooks::Category
# state        public
# description  assign categories from the original product to the cloned product
#========================================================================================
package DE_EPAGES::Catalog::Hooks::Category;

use strict;
use DE_EPAGES::Presentation::API::Constants qw (MAX_PREVIEW_CHILD_ITEMS JQUERY_JSON_ATTRIBUTES);
use DE_EPAGES::Object::API::Factory qw (LoadObject);
use DE_EPAGES::Catalog::API::Object::PagerCategoryProducts;
use DE_EPAGES::Catalog::DAL::CategoryProductMap qw ( DAL_GetProductTreeNodeAttributesByCategoryID );

#========================================================================================
# function     OnJSONChildren
#----------------------------------------------------------------------------------------
# syntax       OnJSONChildren( $hParams );
#----------------------------------------------------------------------------------------
# description  Assign products to json children for content tree.
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object (only categories used) - object
#                   <li>Children - list of child info - ref.array.hash.string
#                   <li>UserLanguageID - id of language used by user. if undef then default shop language
#               </ul> | ref.hash.*
#========================================================================================
sub OnJSONChildren {
    my ($hParams) = @_;
    my $Category = $hParams->{'Object'};
    my $Children = $hParams->{'Children'};
    my $UserLanguageID = $hParams->{'LanguageID'};

    # mark each category as folder if exists products
    foreach my $hChild (@$Children) {
        my $Child = LoadObject($hChild->{'ObjectID'});
        if ($Child->instanceOf('ContentObject')) {
            if ($Child->instanceOf('Category')) {
                $hChild->{'_isCategory'} = 1; # move it or move into it
                $hChild->{'hasChildren'} = 1 if not $hChild->{'hasChildren'} and $Child->get('ProductCount');
            }
            $hChild->{'IsVisibleContentView'} = $Child->get('IsVisibleContentView');
            $hChild->{'SEOQuality'} = $Child->get('SEOQualityAverage') || 0;
            $hChild->{'CanDoSEOCheck'} = $Child->get('CanDoSEOCheck') || 0;
            $hChild->{'VisibleInNavigation'} = $Child->get('VisibleInNavigation');
            $hChild->{'ReadPermissionFor'} = $Child->get('ReadPermissionFor');
            $hChild->{'CanonicalURL'} = $Child->get('CanonicalURL');
        }
    }
    return unless $Category && $Category->instanceOf('Category');
    my $CategoryID = $Category->get('ID');
    my $ahProductTreeNodeAttributes = DAL_GetProductTreeNodeAttributesByCategoryID($CategoryID, $UserLanguageID);
    my $Site = $Category->getSite;
    my $ShopURL = $Site->get('WebUrl');
    my $ShopURLSSL = $Site->get('WebUrlSSL');
    my $ShopPath = $Site->get('Path');
    my $ShopLocaleAlias = $Site->get('SiteLocale')->get('Alias');

    foreach my $row (@$ahProductTreeNodeAttributes) {
        my $hInfo = {};
        $hInfo->{'ObjectID'} = $row->{'objectid'};
        $hInfo->{'Alias'} = $row->{'alias'};
        $hInfo->{'NameOrAlias'} = $row->{'name'} // $row->{'alias'};
        $hInfo->{'ClassID'} = $row->{'classid'};
        $hInfo->{'ClassAlias'} = $row->{'classalias'};
        $hInfo->{'Position'} = $row->{'position'};
        $hInfo->{'Path'} = $ShopPath . '/' .$row->{'alias'};
        $hInfo->{'WebUrl'} = $ShopURL . '/?ObjectPath=' . $hInfo->{'Path'};
        $hInfo->{'WebUrlSSL'} = $ShopURLSSL . '/?ObjectPath=' . $hInfo->{'Path'};
        $hInfo->{'ParentID'} = $CategoryID;
        $hInfo->{'IsVisibleContentView'} = $row->{'isvisible'};
        $hInfo->{'SEOQuality'} = $row->{'quality'};
        $hInfo->{'CanDoSEOCheck'} = 1;
        $hInfo->{'hasChildren'} = $row->{'ismaster'} ? 1 : 0;
        $hInfo->{'_isProduct'} = 1;
        $hInfo->{'CanonicalURL'} = $ShopURL . '/' .  $ShopLocaleAlias . '/?ObjectPath=' . $hInfo->{'Path'};
        $hInfo->{'_isAppointment'} = $Category->get('Alias') eq 'Appointments' ? 1 : 0;
        push @$Children, $hInfo;
    }
    return;
}

sub OnJSONChildrenJSTree {
    my ($hParams) = @_;
    my $Object = $hParams->{'Object'};
    my $hInfo = $hParams->{'Info'};

    return unless $Object;
    if ($Object->instanceOf('Product')) {
        my $Product = $Object;
        my @Children = ();
        my @AttributeAliases = map { $_->{'Attribute'}->alias } @{$Product->get('VariationTypeValues')};
        foreach my $SubProduct (@{$Product->get('SubProducts')}) {
            my $hChild = {
                'data' => {
                    'title'      => join (',', map { $SubProduct->getWithParent($_)->get('ValueOrAlias') } @AttributeAliases),
                    'icon'       => 'Product',
                },
                'metadata' => {
                    'id'         => $SubProduct->id,
                    'parentId'   => $Product->id,
                    'type'       => 'Product',
                }
            };
            $hChild->{'metadata'}  = {%{$hChild->{'metadata'}}, %{$SubProduct->get(JQUERY_JSON_ATTRIBUTES)}};
            push @Children, $hChild;
        }
        $hInfo->{'children'} = \@Children;
    }

    return unless $Object->instanceOf('Category');
    my $Category = $Object;

    if ($hInfo->{'children'}){
        foreach my $hChild (@{$hInfo->{'children'}}){
            my $ChildObject = LoadObject($hChild->{'metadata'}->{'id'});
            next unless $ChildObject->instanceOf('Category');
            if($ChildObject->get('ProductCount') > 0){
                $hChild->{'state'} = 'closed';
            }
        }
    }
    my $ProductCount = $Category->get('ProductCount');
    if($ProductCount > 0 && $ProductCount < MAX_PREVIEW_CHILD_ITEMS){
        foreach my $Product (@{$Category->get('Products')}) {
            my $hChild = {
                'data' => {
                    'title'      => $Product->get('NameOrAlias'),
                    'icon'       => 'Product',
                },
                'metadata' => {
                    'id'         => $Product->id,
                    'parentId'   => $Category->id,
                    'type'       => 'Product',
                }
            };
            if($Product->get('IsMaster')) {
                $hChild->{'state'} = 'closed';
            }
            $hChild->{'metadata'}  = {%{$hChild->{'metadata'}}, %{$Product->get(JQUERY_JSON_ATTRIBUTES)}};
            push @{$hInfo->{'children'}}, $hChild;
        }
    }
}

1;
