#==============================================================================
# §package      DE_EPAGES::Content::API::TLE::ContentObject
# §state        public
#==============================================================================
package DE_EPAGES::Content::API::TLE::ContentObject;

use strict;
use DE_EPAGES::Core::API::Log   qw( GetLog LogError );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Content::API::ShortUrl qw( GetObjectByURI );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );
use MIME::Base64;
use URI::Escape qw( uri_escape_utf8 );
use utf8;

sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'ContentObject.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
}

sub new { bless {}, shift; }

#========================================================================================
# §function     register
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# §description  Registers this handle on the processor
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'ContentObject.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'IsInProductDetailPage');
    $Processor->registerHandler('FunctionHandler', $self, 'GetCanonicalURL');

    return;
}

#========================================================================================
# §function     IsInProductDetailPage
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #IsInProductDetailPage(#Object, #VIEWACTION.Alias)
#----------------------------------------------------------------------------------------
# §description  TLE to determine if a user is on the mbo product page.
#----------------------------------------------------------------------------------------
# §input        $Object, $ViewActionAlias
#----------------------------------------------------------------------------------------
# return        1 or 0 | 1, if customer is on a mbo produtc page, 0 otherwise | int
#========================================================================================
sub IsInProductDetailPage {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Object, $ViewActionAlias) = @$aParams;

    my $Result = eval {
        $self->_isInProductDetailPage($Object, $ViewActionAlias);
    };
    if ( ExistsError() ) {
        LogError(GetError());
        $Result = 0;
    }

    return $Result;
}

#========================================================================================
# §function     GetCanonicalURL
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #GetCanonicalURL(#Object, #LocaleID)
#----------------------------------------------------------------------------------------
# §description  TLE to construct the canonical URL for a given localeID
#----------------------------------------------------------------------------------------
# §input        $Object, $LocaleID
#----------------------------------------------------------------------------------------
# return        $CanonicalUrl | canonical URL added with the given localeID | string
#========================================================================================
sub GetCanonicalURL {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Object, $LocaleID) = @$aParams;

    my $Shop = $Object->getSite();
    my $hInfoShop = $Shop->get([
        'DomainName',
        'LocaleID',
        'HasSSLCertificate',
        'ProtocolAndServerSSL',
        'ProtocolAndServer',
        'WebServerScriptUrlSSL',
        'WebServerScriptUrl'
    ]);
    my $LanguageID = GetInfoLocale($LocaleID)->{'LanguageID'};
    my $URI;

    # use short URLs only in case of a shop domain
    if (defined $hInfoShop->{'DomainName'}) {
        if ($Object->instanceOf('StartPage')) {
            # use www.mydomain/<LocaleID> as short URL for the startpage
            if ($hInfoShop->{'LocaleID'} ne $LocaleID) {
                $URI = $LocaleID;
            } else {
                $URI = '';
            }
            my $CanonicalURL = $hInfoShop->{'HasSSLCertificate'}
                ? $hInfoShop->{'ProtocolAndServerSSL'}
                : $hInfoShop->{'ProtocolAndServer'};
            $CanonicalURL = $CanonicalURL . '/'; # EPG-18932 : ending slash for shop domain
            $CanonicalURL = $CanonicalURL . $URI if $URI ne '';
            return $CanonicalURL;
        } elsif ($Object->instanceOf('Product') && defined $Object->get('SuperProduct')) {
            # get short URL from master product
            $URI = $Object->get('SuperProduct')->get('URI', $LanguageID);
        } else {
            # get short URL from object
            $URI = $Object->get('URI', $LanguageID);
        }
    }

    if (defined $URI) {
        # we have a short URL
        my (undef, $ShortURLLocaleID) = GetObjectByURI($Shop, $URI);
        # return short URL in case we have the correct LocaleID for it
        return $Object->get('CanonicalURL', $LanguageID) if $ShortURLLocaleID eq $LocaleID;
    }
    my $ObjectPath = $Object->instanceOf('Product') && defined $Object->get('SuperProduct')
        ? $Object->get('SuperProduct')->get('Path')
        : $Object->get('Path');
    my $Value = uri_escape_utf8( $ObjectPath );
    $Value =~ s/%2F/\//ig;
    my $CanonicalURL = $hInfoShop->{'HasSSLCertificate'}
        ? $hInfoShop->{'WebServerScriptUrlSSL'}
        : $hInfoShop->{'WebServerScriptUrl'};
    $CanonicalURL = $CanonicalURL . '/' . $LocaleID . '/?ObjectPath=' . $Value;
    return $CanonicalURL;
}


#========================================================================================
# §function     _isInProductDetailPage
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->_isInProductDetailPage($Object, $ViewActionAlias);
#----------------------------------------------------------------------------------------
# §description  Method to determine if a user is on the mbo product page.
#----------------------------------------------------------------------------------------
# §input        $Object, $ViewActionAlias
#----------------------------------------------------------------------------------------
# return        true or false | true, if customer is on a mbo product page | boolean
#========================================================================================
sub _isInProductDetailPage {
    my $self = shift;
    my ($Object, $ViewActionAlias) = @_;

    return 1 if ($Object->instanceOf('Product') && $ViewActionAlias eq 'MBO-ViewGeneral');
    return 0;
}

1;
