#========================================================================================
# package      DE_EPAGES::Design::API::DesignInstaller
# base         DE_EPAGES::Shop::API::ShopInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  Install/uninstall color sets, navigation bars, navigation bar elements.
#               Processes Database/XML/NavBars*.xml files.
#               Processes Database/XML/NavElements*.xml files.
#               Processes Database/XML/MailTypeTemplates*.xml files.
#               Use functions <function DE_EPAGES::Object::API::ObjectInstaller::onPatchDeleteObjects>,
#               <function DE_EPAGES::Object::API::ObjectInstaller::onPatchImportObjects> to patch
#               color sets, navigation bars, navigation bar elements.
#========================================================================================
package DE_EPAGES::Design::API::DesignInstaller;
use base qw( DE_EPAGES::Shop::API::ShopInstaller );

use strict;
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::File qw ( RunOnGlob );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( Transaction );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw( ImportTranslation );
use DE_EPAGES::XML::API::TestSchema qw ( TestSchema );

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->install
#----------------------------------------------------------------------------------------
# description  Installs Database/XML/NavBars*.xml on installation process.
#               Installs Database/XML/NavElementGroups*.xml on installation process.
#               Installs Database/XML/NavElements*.xml on installation process.
#               Installs Database/XML/TemplateTypes*.xml on installation process.
#               Installs Database/XML/StyleTemplates*.xml on installation process.
#               Installs Database/XML/StyleGroups*.xml on installation process.
#========================================================================================
sub install {
    my $self = shift;

    $self->SUPER::install;

    Transaction( sub {
        my $XMLDir = $self->xmlDirectory;
        RunOnGlob($XMLDir, 'Fonts*.xml', sub { ImportObjects( $_, '/' ) });
        RunOnGlob($XMLDir, 'NavBars*.xml', sub { ImportObjects( $_, '/' ) });
        RunOnGlob($XMLDir, 'NavElementGroups*.xml', sub { ImportObjects( $_, '/' ) });
        RunOnGlob($XMLDir, 'NavElements*.xml', sub { ImportObjects( $_, '/' ) });
        RunOnGlob($XMLDir, 'TemplateTypes*.xml', sub { ImportObjects( $_, '/' ) });
        RunOnGlob($XMLDir, 'StyleTemplates*.xml', sub { ImportObjects( $_, '/' ) });
        RunOnGlob($XMLDir, 'StyleGroups*.xml', sub { ImportObjects( $_, '/' ) });
        ImportTranslation( "$XMLDir/Translation.NavBars" );
        ImportTranslation( "$XMLDir/Translation.NavElementGroups" );
        ImportTranslation( "$XMLDir/Translation.NavElements" );
        ImportTranslation( "$XMLDir/Translation.Notrans.NavElements" );
        ImportTranslation( "$XMLDir/Translation.TemplateTypes" );
        ImportTranslation( "$XMLDir/Translation.StyleGroups" );
    });
}

#========================================================================================
# function     test
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test;
#----------------------------------------------------------------------------------------
# description  Runs xml schema validation tests on NavElements
#========================================================================================
sub test {
    my $self = shift;
    $self->SUPER::test();
    my $XMLDir = $self->xmlDirectory;
    my $XSDDir = $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/Design/Data/XSD/';
    TestSchema($XMLDir, "NavElements*.xml", [$XSDDir.'NavElements.xsd', $XSDDir.'v6/NavElements.xsd'] );
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls Database/XML/StyleGroups*.xml on uninstallation process.
#               Uninstalls Database/XML/StyleTemplates*.xml on uninstallation process.
#               Uninstalls Database/XML/TemplateTypes*.xml on uninstallation process.
#               Uninstalls Database/XML/NavElements*.xml on uninstallation process.
#               Uninstalls Database/XML/NavElementGroups*.xml on uninstallation process.
#               Uninstalls Database/XML/NavBars*.xml on uninstallation process.
#========================================================================================
sub uninstall {
    my $self = shift;

    my $XMLDir = $self->xmlDirectory;
    RunOnGlob($XMLDir, 'StyleGroups*.xml', sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force });
    RunOnGlob($XMLDir, 'StyleTemplates*.xml', sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force });
    RunOnGlob($XMLDir, 'TemplateTypes*.xml', sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force });
    RunOnGlob($XMLDir, 'NavElements*.xml', sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force });
    RunOnGlob($XMLDir, 'NavElementGroups*.xml', sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force });
    RunOnGlob($XMLDir, 'NavBars*.xml', sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force });

    $self->SUPER::uninstall;
}

#========================================================================================
# function     invalidateAllStyles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->invalidateAllStyles;
#----------------------------------------------------------------------------------------
# description  Sets the IsInvalid flag for all styles, so the script saveAllStyles.pl
#               will re-generate the CSS stylesheets.
#========================================================================================
sub invalidateAllStyles {
    my $self = shift;

    GetLog()->debug('set isinvalid=1 for all styles');

    my $dbi = GetCurrentDBHandle();
    $dbi->do('UPDATE style SET isinvalid = 1');
}

1;
