#==============================================================================
# package      DE_EPAGES::Design::API::TLE::FontHandler
# state        public
#==============================================================================
package DE_EPAGES::Design::API::TLE::FontHandler;

use strict;
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath ExistsObjectByPath LoadRootObject );
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Design::API::Constants qw (
    NORMAL_FONTSTYLE
    REGULAR_FONTSTYLE
    ITALIC_FONTSTYLE
    DEFAULT_FONTWEIGHT
    @FONT_FILE_EXTENSIONS
);

sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'FontHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
}

sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  Registers this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'FontHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'GetGoogleWebFont');

    return;
}

#========================================================================================
# function     GetGoogleWebFont
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BLOCK("GetGoogleWebFont", #Webfont) <HTML with TLEs> #ENDBLOCK
#----------------------------------------------------------------------------------------
# description  Replaces the properties of all given Google Webfonts in the given HTML
#               block and concatenates the block for all style option
#----------------------------------------------------------------------------------------
# input        #Webfont | webfont string, e.g. "Expletus+Sans:400,700,400italic,700italic" | string
# return       $Content | processed template content for all styles | string
#========================================================================================
sub GetGoogleWebFont {
    my $self = shift;
    my ($Processor, $raParams, $cTemplate) = @_;
    my ($Webfont) = @$raParams;
    TestString( 'Webfont', $Webfont );
    GetLog->debug( "FontHandler.GetGoogleWebFont($Webfont)" );

    my $WebRoot = LoadRootObject()->get('WebRoot');
    my $aFontProperties = $self->_parseWebfont($Webfont);
    my @Content;

    foreach my $hFontProperties (@$aFontProperties) {
        # load related FontStyle object from the store db
        my $FontStylePath = '/Fonts/' . $hFontProperties->{'FontAlias'} . '/Styles/' . $hFontProperties->{'FontStyleAlias'};
        unless (ExistsObjectByPath($FontStylePath)) {
            GetLog->info( "FontHandler.GetGoogleWebFont: Font style '$hFontProperties->{'FontStyleAlias'}' of font family '$hFontProperties->{'FontAlias'}' not found" );
            next;
        }
        my $FontStyle = LoadObjectByPath($FontStylePath);
        my $hInfoFontStyle = $FontStyle->get(['FullName', 'PostScriptName']);

        my $hVars = {
            'Family' => $hFontProperties->{'Family'},
            'Style' => $hFontProperties->{'Style'},
            'Weight' => $hFontProperties->{'Weight'},
            'SVGTag' => $hFontProperties->{'SVGTag'},
            'FullName' => $hInfoFontStyle->{'FullName'},
            'PostScriptName' => $hInfoFontStyle->{'PostScriptName'}
        };

        # add relative web paths for all font files
        foreach my $FileExtension ( @FONT_FILE_EXTENSIONS ) {
            my $FileName = $FontStyle->filePath . '.' . $FileExtension;
            my $FilePath = $WebRoot .'/'. $FileName;
            $hVars->{'URLs'}->{$FileExtension} = $FilePath;
        }

        # store parsed template content
        push @Content, join ('', @{$Processor->replaceTLEs($cTemplate, $hVars)});
    }
    # concatenate parsed template content
    return join ('', @Content);
}

#========================================================================================
# function     _parseWebfont
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->_parseGoogleWebfont($Webfont)
# example      my $aInfoWebFonts = $self->_parseWebfont($Webfont);
#----------------------------------------------------------------------------------------
# description  Parses a given Google webfont string and returns a list of all included
#               web font style properties.
#----------------------------------------------------------------------------------------
# input        $Webfont | webfont string, e.g. "Expletus+Sans:400,700,400italic,700italic" | string
# return       $aInfoWebFonts | list of font properties | ref.array
#========================================================================================
sub _parseWebfont {
    my $self = shift;
    my ($Webfont) = @_;

    # split font family and properties
    my ($Family, $Properties) = split (m/:/, $Webfont);
    # split properties
    my @Properties = split (m/,/, $Properties);
    # use normal font style as default
    @Properties = [NORMAL_FONTSTYLE] unless scalar @Properties;

    # construct different naming properties
    my $FontAlias = lc $Family;
    $FontAlias =~ s/\+/\-/g;
    $Family =~ s/\+/ /g;
    my $SVGTag = $Family;
    $SVGTag =~ s/ //g;

    my @FontProperties;

    foreach my $Property (@Properties) {
        $Property = lc $Property;
        # get weight, if any
        $Property =~ m/^(\d*)/;
        my $Weight = $1;
        # set wweight default value
        $Weight = $Weight ? $Weight : DEFAULT_FONTWEIGHT;
        # get style, if any
        $Property =~ m/([a-z]*)$/;
        my $Style = $1;
        # set style default values
        $Style = $Style ? $Style : NORMAL_FONTSTYLE;
        $Style = ITALIC_FONTSTYLE if $Style eq 'i';

        # generate FontStyle object alias
        my $FontStyleAlias;
        if ($Style eq NORMAL_FONTSTYLE) {
            if ($Weight eq DEFAULT_FONTWEIGHT) {
                $FontStyleAlias = REGULAR_FONTSTYLE;
            } else {
                $FontStyleAlias = $Weight;
            }
        } else {
            if ($Weight eq DEFAULT_FONTWEIGHT) {
                $FontStyleAlias = $Style;
            } else {
                $FontStyleAlias = $Weight . $Style;
            }
        }

        my $FontStyle = {
            'Family' => $Family,
            'Style'  => $Style,
            'Weight'  => $Weight,
            'FontAlias'  => $FontAlias,
            'FontStyleAlias'  => $FontStyleAlias,
            'SVGTag'  => $SVGTag,
        };
        push @FontProperties, $FontStyle;
    }

    return \@FontProperties;
}


1;
