#========================================================================================
# package      DE_EPAGES::Ebay::API::EbayAuction
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the ebay auction functions.
#========================================================================================
package DE_EPAGES::Ebay::API::EbayAuction;
use base Exporter;
use strict;

use DE_EPAGES::Cartridge::API::NonPermTemplate qw ( ProcessFileTemplate );
use DE_EPAGES::Core::API::Error qw (Error GetError ExistsError);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Ebay::API::Constants qw ( AUCTION_EDITABLE
                                         UNITY_DEFAULT_TEMPLATE
                                         UNITY_TEMPLATE_PATH
                                         UNITY_DEFAULT_PICTURE_TYPE
                                         UNITY_DEFAULT_PIC_VARIATION_TYPE );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use feature qw ( switch );

our @EXPORT_OK = qw (
    InsertNewAuctionFromProduct
);

#========================================================================================
# function     InsertNewAuctionFromProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       InsertNewAuctionFromProduct( $Shop, $Product );
#----------------------------------------------------------------------------------------
# description  Inserts new auction from product.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $Product | product object | object
# input        $ParentAuction | (optional) ParentAuction object | object
# input        $hBusinessPolicySellerProfiles | (optional) business seller profiles | ref.hash
#----------------------------------------------------------------------------------------
# return        $Auction | auction object | object
#========================================================================================
sub InsertNewAuctionFromProduct { ##no critic qw(ProhibitExcessComplexity)
    my ($Shop, $Product, $ParentAuction, $hBusinessPolicySellerProfiles) = @_;

    my $IsUnityRegistered = $Shop->get('IsUnityRegistered');

    my $Folder = $Shop->folder('EbayAuctions', { Position=> 190 });
    my $DefaultAuction = $Shop->get('EbayDefaultAuction');

    my $ShopEnv = $DefaultAuction->get('EbayShopEnvironment');
    Error('NoShopEnvironment') unless $ShopEnv;

    my $Env = $ShopEnv->get('EbayEnvironment');

    if ($Product->get('IsDownloadProduct')) {
        Error('CanNotAuctionDownloadProduct', {'ProductID', $Product->id, 'Product' => $Product});
    }

    #check if currency id is available
    my $Currencies = $Shop->get('Currencies');
    my $currencyExists = 0;
    foreach my $cur (@$Currencies) {
        $currencyExists = 1 if ($cur->{'CurrencyID'} eq $Env->get('CurrencyID'));
    }
    if (not $currencyExists) {
        Error('EbaySiteNeedsCurrencyThatIsNotEnabledOnShop', {
            'EbayCurrency' => $Env->get('CurrencyID'),
            'ShopCurrencies' => join (', ', map {$_->{'CurrencyID'}} @$Currencies),
        });
    }

    my $Price;
    eval {
        # get price from product
        $Price = $Product->price({
            'CurrencyID'    => $Env->get('CurrencyID'),
            'TaxModel'      => $Shop->get('TaxModel'),
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'ProductNoPrice') {
            $Price = 0;
        } else {
            Error($Error);
        }
    }

    # get lang of ebay site
    my $langid = GetPKeyLanguageByCode(lc $Env->get('LanguageID'));
    # use default shop lang if ebay site's lang isn't active in the shop
    $langid = undef unless $langid ~~ [ map {$_->{'LanguageID'}} @{$Shop->get('Languages')} ];

    my $name = $Product->getWithParent('NameOrAlias', $langid) ||
               $Product->getWithParent('NameOrAlias');

    my $descr = '';
    my $ProductWebPath = $Product->get('WebPath');
    # load default template for Unity shops from system - UNITY-2477, use default shop template otherwise
    if ($IsUnityRegistered) {
        my $DefaultTemplate = LoadObjectByPath(UNITY_DEFAULT_TEMPLATE);
        my $TemplateName = $DefaultTemplate->alias.'.html';
        my $ThemePath = UNITY_TEMPLATE_PATH . '/' . $TemplateName;
        my $hVars = { 'Product'             => $Product->tleHash,
                      'TextBelowUnityTheme' => $DefaultAuction->get('TextBelowUnityTheme'),
                      'ImagePath'           => $ProductWebPath };
        $descr = ProcessFileTemplate($ThemePath, $hVars);
    } else {
        $descr = $DefaultAuction->get('EbayUserTemplate')->process( $Product, $langid ) if ($DefaultAuction->get('EbayUserTemplate'));
    }
    $descr = $Product->getWithParent('Text', $langid) ||
             $Product->getWithParent('Description', $langid) ||
             $Product->getWithParent('Text') ||
             $Product->getWithParent('Description') unless $descr;

    my $EbayStoreCategoryId = $DefaultAuction->get('EbayStoreCategoryId');
    $EbayStoreCategoryId = undef if ( defined $EbayStoreCategoryId
                                        && ( $EbayStoreCategoryId eq 'dummy'
                                            || $EbayStoreCategoryId   == 0 ) );

    my $EbayStoreCategory2Id = $DefaultAuction->get('EbayStoreCategory2Id');
    $EbayStoreCategory2Id = undef if ( defined $EbayStoreCategory2Id
                                        && ( $EbayStoreCategory2Id eq 'dummy'
                                            || $EbayStoreCategory2Id   == 0 ) );

    my $params = {
        'Alias'             => $Folder->nextAlias('EbayAuction%s'),
        'Parent'            => $Folder,
        'Product'           => $Product,
        'Status'            => AUCTION_EDITABLE,
        'ExternalAuctionId' => undef,
        'Name'              => $name,
        'Description'       => $descr,
        'CurrentBid'        => 0,
        'BidCount'          => 0,
        'StartTime'         => GetCurrentDBHandle()->currentDateTime,
        'EbayStoreCategoryId'  => $EbayStoreCategoryId,
        'EbayStoreCategory2Id' => $EbayStoreCategory2Id,
    };

    my $ProtocolServerSSLName = $Product->getSite()->get('ProtocolAndServerSSL');

    if (defined $DefaultAuction->get('PictureUrl') and $DefaultAuction->get('PictureUrl') =~ /\S/) {
        my $image;
        if  ($DefaultAuction->get('PictureUrl') eq 'ListView') {
            $image = $Product->getWithParent('ImageSmall');
        } elsif ($DefaultAuction->get('PictureUrl') eq 'DetailView') {
            $image = $Product->getWithParent('ImageMedium');
        } elsif ($DefaultAuction->get('PictureUrl') eq 'ZoomView') {
            $image = $Product->getWithParent('ImageLarge');
        } elsif ($DefaultAuction->get('PictureUrl') eq 'HotDealView') {
            $image = $Product->getWithParent('ImageHotDeal');
        }

        if (not defined $image) {
            # no image.
        } elsif ($image =~ /\/\//) { #contains //hostname
            $params->{'PictureUrl'} = $image;
        } else {
            $params->{'PictureUrl'} = $ProtocolServerSSLName . $ProductWebPath . '/' . $image;
        }
    } elsif ($IsUnityRegistered) {
        # set default image for Unity shops
        $params->{'PictureUrl'} = $ProtocolServerSSLName . $ProductWebPath
            . '/' . $Product->get(UNITY_DEFAULT_PICTURE_TYPE);
    }

    if (defined $DefaultAuction->get('GalleryUrl') and $DefaultAuction->get('GalleryUrl') =~ /\S/) {
        my $image;
        if  ($DefaultAuction->get('GalleryUrl') eq 'ListView') {
            $image = $Product->getWithParent('ImageSmall');
        } elsif ($DefaultAuction->get('GalleryUrl') eq 'DetailView') {
            $image = $Product->getWithParent('ImageMedium');
        } elsif ($DefaultAuction->get('GalleryUrl') eq 'ZoomView') {
            $image = $Product->getWithParent('ImageLarge');
        } elsif ($DefaultAuction->get('GalleryUrl') eq 'HotDealView') {
            $image = $Product->getWithParent('ImageHotDeal');
        }

        if ($image =~ /\/\//) { #contains //hostname
            $params->{'GalleryUrl'} = $image;
        } else {
            $params->{'GalleryUrl'} = $ProtocolServerSSLName .  $ProductWebPath . '/' . $image;
        }
    } elsif ($IsUnityRegistered) {
        # set default gallery image
        my $Gallery = $DefaultAuction->get('Gallery');
        my $GalleryImage;
        given ($Gallery) {
            when ('Gallery') {
                $GalleryImage = $Product->getWithParent('ImageMedium');
            }
            when('Plus') {
                $GalleryImage = $Product->getWithParent('ImageLarge');
            }
        }
        $params->{'GalleryUrl'} = $ProtocolServerSSLName . $ProductWebPath . '/' . $GalleryImage
            if defined $GalleryImage;
    }

    if ($DefaultAuction->get('ListingType') eq 'FixedPriceItem') {
        $params->{'StartPrice'} = $Price;
    } else {
        $params->{'BuyItNowPrice'} = $Price;
    }

    if (defined $ParentAuction) {
        $params->{'ParentAuction'} = $ParentAuction;
        $params->{'StartPrice'} = $Price;
    }

    if($Product->get('IsMaster')){
         $params->{'ListingType'} = 'FixedPriceItem';
         # set default type for variations
         $params->{'PicVariationType'} = $IsUnityRegistered
            ? UNITY_DEFAULT_PIC_VARIATION_TYPE : $DefaultAuction->get('PictureUrl');
    }

    if(defined $ParentAuction){
        $params->{'Quantity'} = (($Product->get('StockLevel') // 0)) > 0 ? $Product->get('StockLevel') : 0;
    }

    my $return = $DefaultAuction->clone( $params, $hBusinessPolicySellerProfiles );

    if($Product->get('IsMaster')){
        my $MasterQuantity = 0;
        foreach my $SubProduct ( @{ $Product->folder('SubProducts')->children } ) {
            if($SubProduct->get('IsVisible')) {
                InsertNewAuctionFromProduct($Shop,$SubProduct, $return, $hBusinessPolicySellerProfiles) ;
                $MasterQuantity += (($SubProduct->get('StockLevel') // 0)) > 0 ? $SubProduct->get('StockLevel') : 0;
            }
        }
        $return->set({'Quantity' => $MasterQuantity});
    }

    return $return;

}

1;
