#========================================================================================
# package      DE_EPAGES::GoogleAdWords::API::TLE::GoogleAdWordsHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Provides TLE function for localisation of button url
#========================================================================================
package DE_EPAGES::GoogleAdWords::API::TLE::GoogleAdWordsHandler;

use strict;
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Presentation::API::SiteTLEProcessor qw (ReplaceTLEs);

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function register this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'GoogleAdWordsHandler.register' );
    $Processor->registerHandler('FunctionHandler', $self, 'ShopGoogleAdWordsParameter');
    return;
}

#========================================================================================
# function     ShopGoogleAdWordsParameter
# state        public
#----------------------------------------------------------------------------------------
# syntax       #FUNCTION("ShopGoogleAdWordsParameter", #Object, #VIEWACTION.Object)
#----------------------------------------------------------------------------------------
# example      #LOCAL("ShopGoogleAdWordsParameter", #FUNCTION("ShopGoogleAdWordsParameter",
#               #Object, #VIEWACTION.Object))
#               #IF(#ShopGoogleAdWordsParameter.IsActive) ...
#----------------------------------------------------------------------------------------
# description  This function provides an info if google adwords tracking
#               javascript code should be inserted on the page.
#----------------------------------------------------------------------------------------
# input        $Object | current view object | object
# input        $ViewAction | viewaction object | object
# return       $hInfo - used keys are:
#               <ul>
#                 <li>IsOrder - 1 if this is an order confirmation page, else 0</li>
#               </ul>
#========================================================================================
sub ShopGoogleAdWordsParameter {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Object, $ViewAction) = @$aParams;

    my %Info;
    my $ViewActionAlias = $ViewAction->alias;
    # Set IsOrder=1 only on the order confirmation page
    if ( $Object->instanceOf('CustomerOrder') and ($ViewActionAlias eq 'View' or $ViewActionAlias eq 'MobileSF-View' or $ViewActionAlias eq 'ViewMultiCheckoutCustomerOrder') ) {
        $Info{'IsOrder'} =  1;
        # EPG-23599: Replace own TLE #OrderAmount with the real order amount (without currency).
        my $GoogleAdWordsHTMLCode = $Object->getSite()->get('GoogleAdWordsHTMLCode');
        # add [0] formatter to avoid JS escaping (AD-8794)
        $GoogleAdWordsHTMLCode =~ s/\#OrderAmount(?!\[.+\])/\#OrderAmount\[0\]/g;
        $Info{'GoogleAdWordsCode'} = ReplaceTLEs(
            $GoogleAdWordsHTMLCode,
            {'OrderAmount' => $Object->child('LineItemContainer')->get('GrandTotal')}
        );
    }

    return \%Info;
}

1;
