#========================================================================================
# package      DE_EPAGES::Presentation::API::TLE::IncludeTemplateHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Provides TLE functions and variables for partial page caching.
#               The TLE block <function CachedInclude> can be used to cache a part of
#               a page that would normally not be cached. This is useful for content
#               areas that are mostly static but take a long time to compute.
#               If the content is not completely static, you must specify a cache file
#               name that includes the parameter on which the content depends. For
#               example, if the content depends on the language and the currency,
#               the file name should include the current currency is and language id.
#               There are some pre-defined file name parts for most common scenarios,
#               such as #CacheIncludesPrices for page parts that contain locale
#               and currency dependend content. See function <function tle> for details.
# example      #BLOCK("CachedInclude", #Object, "MyPageType.MyInclude" .  #INPUT.Language . #INPUT.Currency)
#                   #NameOrAlias: #Price[money]
#               #ENDBLOCK
# example      #BLOCK("CachedInclude", #Object, "MyPageType.MyInclude" .  #CacheIncludesPrices)
#                   #NameOrAlias: #Price[money]
#               #ENDBLOCK
#========================================================================================
package DE_EPAGES::Presentation::API::TLE::IncludeTemplateHandler;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Object::API::PageCache;
use DE_EPAGES::Permission::API::Permission qw (ActionByName);
use DE_EPAGES::Presentation::API::Template qw (ProcessPage GetTLEProcessor);
use DE_EPAGES::Presentation::API::Object::ObjectPageType;
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByViewAction);
use DE_EPAGES::WebInterface::API::ApplicationServer qw (GetAppServer);
use DE_EPAGES::Core::API::PerlTools qw (trim);

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function register this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'IncludeTemplateHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'IncludeTemplateHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'IncludeTemplateViaObject');
    $Processor->registerHandler('FunctionHandler', $self, 'CachedInclude');
    $Processor->registerHandler('FunctionHandler', $self, 'SavedInclude');

    $Processor->registerHandler('FunctionHandler', $self, 'LocaleInclude');
    $Processor->registerHandler('FunctionHandler', $self, 'InlineEditor');
    $Processor->registerHandler('FunctionHandler', $self, 'SNIPPET');

    $Processor->registerHandler('VariableHandler', $self, 'CacheIncludesNames');
    $Processor->registerHandler('VariableHandler', $self, 'CacheIncludesPager');
    return;
}

#========================================================================================
# function     IncludeTemplateViaObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       #IncludeTemplateViaObject(#Object, #Template, #ViewAction)
# example      #BLOCK("Pager", #VisibleProducts, 1)
#                 #LOOP(#Pager.Items)
#                   #IncludeTemplateViaObject(#Object, "Content", "View")
#                 #ENDLOOP
#                 <div class="PagedProduct">#INCLUDE("Pager")</div>
#               #ENDBLOCK
#----------------------------------------------------------------------------------------
# description  Includes a template from a different page type.
#               This allows to show the content of a product detail page in the context
#               of a category page.
#----------------------------------------------------------------------------------------
# input        #Object | object to display within the block | object
# input        #Template | name of an include template | string
# input        #ViewAction | name of a ViewAction that is valid for #Object | string
#========================================================================================
sub IncludeTemplateViaObject { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($aProcessor, $raParams, $cTemplate) = @_;
    my $Object = $raParams->[0];
    my $TemplateName = $raParams->[1];
    my $ActionName = $raParams->[2];

    my $Action = ActionByName($Object->class, $ActionName );
    my $PageType = LoadPageTypeByViewAction( $Action, $Object );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Object );

    my $Processor = GetTLEProcessor('IncludeTemplatePageProcessor');

    my $LanguageIdentifier = $aProcessor->getContext()->{'LanguageIdentifier'};
    my @LanguageArray = split(/-/, $LanguageIdentifier);
    my $LanguageCode = $LanguageArray[0];

    my $Vars = $Object->tleHash;
    $Vars->{'PAGETYPE'} = $PageType->tleHash;
    $Vars->{'VIEWACTION'} = $Action->tleHash;
    $Vars->{'INPUT'} = $aProcessor->tle('INPUT');
    return ProcessPage($Object, $ObjectPageType, $LanguageCode, $Vars, $Processor, undef, $TemplateName);
}

#========================================================================================
# function     CachedInclude
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BLOCK("CachedInclude", #Object, #FileName) ... #ENDBLOCK
# syntax       #BLOCK("CachedInclude", #Object, #FileName, #Condition) ... #ENDBLOCK
# example      #BLOCK("CachedInclude", #Object, "SF.CategoriesLinkList" . #CacheIncludesNames)
#                 #LOOP(#Shop.Categories.VisibleChildObjects)
#                   <div><a href="?ObjectPath=#Path[url]">#NameOrAlias</a></div>
#                 #ENDLOOP
#               #ENDBLOCK
#----------------------------------------------------------------------------------------
# description  Caches the content of the block in a static file.
#               The cached content can be reused in other templates if the object and
#               file name are equal. The function uses cache settings of the site to
#               invalidate the stored page.
#----------------------------------------------------------------------------------------
# input        #Object | object that is displayed in the block. The cache file
#                       (#FileName) will be stored in the cache directory of this object
#                       (#Object.StaticPath)  | object
# input        #FileName | cache file name | string
# input        #Condition | (optional, default=1) the content is cached only if this
#                       condition is true | boolean
#========================================================================================
sub CachedInclude { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $raParams, $cTemplate) = @_;

    my $DebugText;
    my $Content;
    my $Condition = scalar @$raParams > 2 ? $raParams->[2] : 1;

    # form error ?
    my $Servlet = GetAppServer->currentServlet;
    if ($Servlet->vars('FormError')) {
        $Content = join '', @{$Processor->replaceTLEs($cTemplate)};
        $DebugText = 'FormError';
    } elsif ($Servlet->isEditorMode) {
        $DebugText = 'SF at BackOffice';
        $Content = join '', @{$Processor->replaceTLEs($cTemplate)};
    } elsif ( !$Condition ) {
        $DebugText = 'disabled by condition';
        $Content = join '', @{$Processor->replaceTLEs($cTemplate)};
    } else {
        # build file name
        my $Object        = $raParams->[0];
        my $FileName      = $raParams->[1];

        my $SitePageCache = DE_EPAGES::Object::API::PageCache->new( Site => $Object->getSite );

        my $LanguageIdentifier = $Processor->getContext()->{'LanguageIdentifier'};
        my $CacheKey = 'INC-'.$FileName.'_'.$LanguageIdentifier;

        ($Content, undef) = $SitePageCache->compute( $Object, $CacheKey, undef, sub {
            my $IsCacheable = $Object->pageCacheDuration > 0 && !$Servlet->cookiesAtPath;
            return join('', @{$Processor->replaceTLEs($cTemplate)}), $IsCacheable;
        });

        $DebugText = $CacheKey;
    }

    return GetLog->is_debug()
        ? "<!-- CachedInclude $DebugText -->$Content<!-- /CachedInclude -->"
        : $Content;
}

#========================================================================================
# function     LocaleInclude
# state        public
#----------------------------------------------------------------------------------------
# syntax       #LocaleInclude(#Template, #Locale)
# example      #WITH(#Shop.Address)
#                 #LocaleInclude("ShopAddress", #Country.Code2)
#               #ENDWITH
#----------------------------------------------------------------------------------------
# description  Includes a template with locale-specific content.
#               The function implements a fallback mechanism:
#               If the template "#Template-#Locale" (e.g. "ShopAddress-DE") does not
#               exist, then it includes the template without the locale extension
#               (e.g. "ShopAddress").
#----------------------------------------------------------------------------------------
# input        #Template | template name | object
# input        #Locale | locale string, such as "DE" or "de_DE" | string
#========================================================================================
sub LocaleInclude { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $raParams, $cTemplate) = @_;
    my ($Template, $Locale) = @$raParams;

    my $PageType = $Processor->getContext()->{'PageType'}->get('PageType');
    if( defined $Locale ) {
        my $LocaleTemplate = $Template .'-'. $Locale;
        $Template = $LocaleTemplate if $PageType->existsTemplate( $LocaleTemplate );
    }

    return $Processor->callBlock( 'INCLUDE', sub { return [ $Template ] } );
}

#========================================================================================
# function     SavedInclude
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BLOCK("SavedInclude", #Object, #FileName) ... #ENDBLOCK
# example      #BLOCK("SavedInclude", #Site.Object, "MBO.Menu" . #CacheIncludesNames)
#                   menucontroller.registerChildren([
#                 #MENU("Menu")
#                   #INCLUDE(#Template),
#                 #ENDMENU
#                   null]);
#               #ENDBLOCK
#----------------------------------------------------------------------------------------
# description  Caches the content of the block in a static file.
#               The cached content can be reused in other templates if the object and
#               file name are equal. The function ignores the cache settings of the site.
#               See also: <function DE_EPAGES::Object::API::Object::Object::clearSavedPages>
#----------------------------------------------------------------------------------------
# input        #Object | object that is displayed in the block. The cache file
#                       (#FileName) will be stored in the static/store../SavedIncludes directory
#                       | object
# input        #FileName | cache file name | string
#========================================================================================
sub SavedInclude { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $raParams, $cTemplate) = @_;
    my ($Object,$FileName) = @$raParams;
    return $Object->savePage(
        $FileName,
        sub{
            return join '', @{$Processor->replaceTLEs($cTemplate)};
        }
    );
}

#========================================================================================
# function     InlineEditor
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BLOCK("InlineEditor", #Type, #Object, #Attribute, #Info)Template#ENDBLOCK
# example      #BLOCK("InlineEditor", "html", #Object, "Description", "Tag=SPAN;Class=Description")
#                 #IF(#Description)
#                 <div class="Description">#Description[0]</div>
#                 #ENDIF
#               #ENDBLOCK
#----------------------------------------------------------------------------------------
# description  Includes a inline editor for different types of data if the template will
#               be processed at the Backoffice.
#               The "Info" parameter is used to supply additional variables, i.e. for
#               #BLOCK("InlineEditor", "html", #Object, "Description", "Tag=span")
#                   ...
#               #ENDBLOCK
#----------------------------------------------------------------------------------------
# input        #Type | type of the inline editor, i.e. text,html,image,... | string
# input        #Object | object to show the inline editor for | object
# input        #Attribute | object's attribte to show the inline editor for | string
# input        #Info | tle variables for replacement, formated like "Key1=Val1;Key2=Val2;.." | string
#========================================================================================
sub InlineEditor { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $raParams, $cTemplate) = @_;
    my ($Type, $Object, $AttributeAlias, $Info) = @$raParams;
    my %editorVars = defined $Info ? $Info =~ /[^=;]+/g : ();

    my $INPUT = $Processor->tle('INPUT');
    if ($INPUT->{'IsEditorMode'}) {
        # SF preview in MBO
        my $Vars = $Object->tleHash;
        my $Attribute = $Object->class->attribute($AttributeAlias);
        if (not defined $Attribute) {
            Error('UNKNOWN_ATTRIBUTE', {
                Attribute => $AttributeAlias,
                ObjectID => $Object->id,
                ObjectAlias => $Object->alias,
                ObjectClass => $Object->class->alias,
            });
        }
        $editorVars{'Type'} = $Type;
        $editorVars{'Attribute'} = $Attribute->tleHash;
        $editorVars{'Value'} = $Object->tleHash->{$AttributeAlias};
        $editorVars{'Value'} = '' unless defined $Object->tleHash->{$AttributeAlias};
        $editorVars{'Tag'} = 'span' unless defined $editorVars{'Tag'};
        $editorVars{'ShowPageBreak'} = 'false' unless defined $editorVars{'ShowPageBreak'};
        $editorVars{'AttributeName'} = $Attribute->get('NameOrAlias') . ' (' . $Object->class->get('NameOrAlias') . ')';
        $Vars->{'InlineEditor'} = \%editorVars;
        $editorVars{'Value'} = _ReplaceGadgets($editorVars{'Value'});
        # create inline editor
        return $Processor->callBlock( 'INCLUDE', sub { [ 'InlineEditor-'.(lc $editorVars{'Type'}), undef, $Vars ] });
    } else {
        # SF
        my $Content = join '', @{$Processor->replaceTLEs($cTemplate)};
        $Content = _ReplaceGadgets($Content) if $INPUT->{'IsEditorNavElements'};
        return $Content;
    }
}

sub _ReplaceGadgets {
    my ($String) = @_;

    # replace gadget div with placeholder img with new comment markup
    ## no critic (RegularExpressions::ProhibitComplexRegexes)
    $String =~
        s/<!--\s*BEGIN_GADGET\s*-->[\s\S]*?<div\s+(class=\"epItemContent\"\s+ep_info=\"|class=\"epItemContent\sep_info:)(.+?)\">[\s\S]*?<!--\s*END_GADGET\s*-->
         /<div dojoType="epages.cartridges.de_epages.presentation.widget.EditorPlaceholder" ep_info="$2"><\/div>/isgx; #"
    ## use critic

    if( $String =~ /<div\s+class=\"epItemContent\"\s+ep_info=\"(.*?)\"\s*>/ ) { #"
        GetLog()->debug( 'epItemContent without BEGIN_GADGET found' );
        $String = _ReplaceDivs( $String );
    }

    return $String;
}

# For compatibility with ePages 6.0.1 try to find the matching </div>
# without a recursive RegExp pattern. This will fail if the epItemContent
# div inludes nested div tags
sub _ReplaceDivs {
    my ($String) = @_;

    # replace gadget without markup, but only the start tag (EPG-8588)
    my $NewString = '';
    EP_INFO: while ($String =~ /<div\s+class\s*=\s*\"epItemContent\"\s+ep_info=\"(.*?)\"\s*>/p) { #"
        $NewString .= ${^PREMATCH};
        my $EpInfo = $1;
        my $AfterDiv = ${^POSTMATCH};
        my $Found = 0;

        # find </div>
        DIV: while( 1 ) {

            my $nextStart = $AfterDiv =~ /<div\b/p ? length(${^PREMATCH}) : -1;
            my $nextEmpty = $AfterDiv =~ /<div\s*\//p ? length(${^PREMATCH}) : -1;

            if( $nextEmpty >= 0 && $nextEmpty <= $nextStart ) {
                # skip empty tag
                $AfterDiv = ${^POSTMATCH};
                next DIV;
            }

            my $nextEnd = $AfterDiv =~ /<\/div>/p ? length(${^PREMATCH}) : -1;

            if( $nextEnd >= 0 ) {
                # end tag found
                $AfterDiv = ${^POSTMATCH};
                if( $nextStart < 0 || $nextStart > $nextEnd ) {
                    # end tag found before start tag
                    $Found = 1;
                    last DIV;
                }
            }
            else {
                # no end tag found
                last;
            }
        }

        $NewString .= '<div dojoType="epages.cartridges.de_epages.presentation.widget.EditorPlaceholder" ep_info="'.$EpInfo.'"></div>';

        # replace everything after the start tag if we can't find the end tag
        $String = $Found ? $AfterDiv : '';
    }

    return $NewString . $String;
}

#========================================================================================
# function     SNIPPET
# state        public
#----------------------------------------------------------------------------------------
# syntax       #SNIPPET("IncludeTemplate", #KeyString, #Value1, #Value2, ...)
# example      #SNIPPET("INC-WYSIWYG", "Name,Value,Head,Help", "Description", #Description, #TRANSLATE("Description"), #TRANSLATE("BubbleHelpDescription"))
#               same as
#               #LOCAL("Name", "Description")#LOCAL("Value", #Description)
#               #LOCAL("Head", #TRANSLATE("Description"))#LOCAL("Help", #TRANSLATE("BubbleHelpDescription"))
#               #INCLUDE("INC-WYSIWYG")
#               #ENDLOCAL#ENDLOCAL#ENDLOCAL#ENDLOCAL
#               #SNIPPET("INC-WYSIWIG", "Name", "Description", "NoDebug")
#----------------------------------------------------------------------------------------
# description  SNIPPET combines LOCAL and INCLUDE to one function. WithDebugInfo strings can be:
#               "NoDebug", "JsDebug", "CssDebug" or "HtmlDebug".
#----------------------------------------------------------------------------------------
# input        #IncludeTemplate | template name | string
# input        #KeyString | comma seperated keys (optional) | string
# input        #Value | list of values | *
# input        #WithDebugInfo | type of the debug info to include | string
#========================================================================================
sub SNIPPET { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $raParams, $cTemplate) = @_;
    my ($Template, $KeyString, @Parameter) = @$raParams;
    $KeyString = '' unless defined $KeyString;

    my @RawKeys = split /,/, $KeyString;
    my @Keys = trim(@RawKeys);

    my $CountContext = scalar @{$Processor->{'Stack'}};
    local $Processor->{'SetTLEIndex'}{$_} = $CountContext foreach @Keys;

    my $rContext = $Processor->{'Stack'}[0];

    my $OldValue = exists $rContext->{'SNIPPET'} ? $rContext->{'SNIPPET'} : undef;

    $rContext->{'SNIPPET'} = {};
    for(my $i = 0; $i < scalar @Keys; $i++) {
        $rContext->{'SNIPPET'}->{$Keys[$i]} = $Parameter[$i];
    }

    # hand over debug info as last parameter, if any
    my $WithDebugInfo;
    $WithDebugInfo = $Parameter[scalar @Keys] if scalar @Parameter > scalar @Keys;

    my $result = $Processor->callBlock( 'INCLUDE', sub { return [ $Template, $WithDebugInfo ] } );

    if (defined $OldValue) {
        $rContext->{'SNIPPET'} = $OldValue;
    } else {
        delete $rContext->{'SNIPPET'};
    }

    return $result;
}

#========================================================================================
# function     tle
# state        private
#----------------------------------------------------------------------------------------
# syntax       #BLOCK("CachedInclude", #Object, #Prefix . #CacheIncludesNames)
# example      #BLOCK("CachedInclude", #Object, "Name" . #CacheIncludesNames)
#                 #NameOrAlias
#               #ENDBLOCK
#----------------------------------------------------------------------------------------
# description  These TLE variables can be used to create special file names for partial
#               page caching with <function CachedInclude> blocks.
#               The generated file names make sure that a unique file is used when the
#               cached context depends on some context, for example if different content
#               would be generated depending on the selected language or currency.
#               <ul>
#                 <li>CacheIncludesNames - content depends on language only - string</li>
#                 <li>CacheIncludesPager - content includes a pager and therefore
#                   depends on Page, OrderBy, OrderDesc, PageSize and PagerSize - string</li>
#               </ul>
#========================================================================================
sub tle { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $TLEName) = @_;

    my $INPUT = $Processor->tle('INPUT');
    my @VARS;
    if( $TLEName eq 'CacheIncludesNames' )          { @VARS = qw( Language ); }
    elsif( $TLEName eq 'CacheIncludesPager' )       { @VARS = qw( Page OrderBy OrderDesc PageSize PagerSize ); }
    else { return undef; }

    return '_' . join ('_', map { defined($_) ? $_ : '' } map { $INPUT->{$_} } @VARS);
}

1;
