﻿#========================================================================================
# §package      DE_EPAGES::Product::API::Cartridge
# §base         DE_EPAGES::ShopMailType::API::ShopMailTypeInstaller;
# §state        public
#----------------------------------------------------------------------------------------
# §description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Product::API::Cartridge;
use base DE_EPAGES::ShopMailType::API::ShopMailTypeInstaller;
use strict;
use utf8;

use DE_EPAGES::Core::API::Log qw(GetLog);
use DE_EPAGES::Database::API::Connection qw(GetCurrentDBHandle);
use DE_EPAGES::Dictionary::API::XML::TranslationImportHandler;
use DE_EPAGES::Object::API::Factory qw(LoadClassByAlias LoadObject Transaction);
use DE_EPAGES::Object::API::Table::Language qw( GetAllInfoLanguages );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw( ImportTranslation );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Presentation::API::PageType qw( LoadPageTypeByAlias );

#========================================================================================
# §function     new
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer = DE_EPAGES::Product::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# §description  Creates a new installer object of DE_EPAGES::Product.
#----------------------------------------------------------------------------------------
# §input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# §return       $Installer | cartridge installer | object
# §todo_patch   update attribute set exportlevel = '1' where isexportable=1 and isuserdefined = 1
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Product',
        'Version'            => '7.25.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0'], ## no critic qw(ProhibitLongLines)
    );
    return bless $self, $class;
}

#========================================================================================
# §function     install
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# §description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;

    ImportObjects( $self->xmlDirectory .'/CustomizableProducts.xml' );
    ImportObjects( $self->xmlDirectory .'/EcoParticipationCategories.xml' );
    ImportTranslation( $self->xmlDirectory . '/Translation.Notrans.EcoParticipationCategories' );

    $self->finishInstall;
    return;
}

#========================================================================================
# §function     uninstall
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# §description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;

    $self->call( sub { DeleteObjects( $self->xmlDirectory . '/EcoParticipationCategories.xml' ) } ) ;

    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

# build SQL statement for each language and attribute
# execute statement (update and insert if not exists)
sub patchStockLevelTexts_6_13_4 {
    my $self = shift;
    my ($hStructure) = @_;

    my $dbi = GetCurrentDBHandle();

    # get attributeIDs
    my %AttributeIDs = map { $_ => $self->getAttributeID( 'Shop', $_ ) } qw (
        ProductLongerDeliveryTimeText ProductNoAddToBasketText
        ProductNoStockDataText ProductStockCriticalText ProductStockAvailableText
        ProductLowOnStockAction
    );

    # get all Languages
    my $ahLanguages = [ grep { $_->{'IsUsed'} } @{GetAllInfoLanguages()} ];

    GetLog()->info('start update or insert new availability texts for EPG-4295');

    # start transaction for all attribute changes
    Transaction ( sub {

        my $sql;
        foreach my $hLanguage (@$ahLanguages) {

            my $LanguageCode = $hLanguage->{'Code2'};
            my $LanguageID = $hLanguage->{'LanguageID'};
            my $Dialect = $hLanguage->{'Dialect'};

            $LanguageCode .= "-$Dialect" if $Dialect;

            next unless exists $hStructure->{$LanguageCode};

            # for each Attribute to set
            foreach my $Attributename (keys %AttributeIDs) {

                # ignore ProductLowOnStockAction beacuse we read this attribute only
                next if $Attributename eq 'ProductLowOnStockAction';

                # get value from structure
                my $AttributeValue = $hStructure->{$LanguageCode}->{$Attributename};
                die "Missing value for $Attributename/$LanguageCode" unless $AttributeValue;

                # special handling for ProductLongerDeliveryTimeText (values depend on ProductLowOnStockAction )
                if ($Attributename eq 'ProductLongerDeliveryTimeText') {

                    GetLog()->info("update $Attributename (ProductLowOnStockAction!=1) for language $LanguageCode");

                     # update ProductLongerDeliveryTimeText for ProductLowOnStockAction != 1
                    $sql = <<'END_SQL';
UPDATE localizedstringattribute
SET value = ?
WHERE languageid = ?
  AND attributeid = ?
  AND objectid NOT IN (
    SELECT objectid
    FROM integerattribute
    WHERE attributeid = ?
    AND value = 1
  )
END_SQL

                    ## no critic qw(ProhibitLongLines)
                    #print "VALUE= $AttributeValue ,  LID: $LanguageID , $Attributename ID: $AttributeIDs{$Attributename}  , ProductLowOnStockAction ID:$AttributeIDs{'ProductLowOnStockAction'}\n";
                    GetLog()->info("update VALUE= $AttributeValue ,  LID: $LanguageID , $Attributename ID: $AttributeIDs{$Attributename}  , ProductLowOnStockAction ID:$AttributeIDs{'ProductLowOnStockAction'}");
                    ## use critic

                    $dbi->do( $sql, [
                        $AttributeValue,
                        $LanguageID,
                        $AttributeIDs{$Attributename},
                        $AttributeIDs{'ProductLowOnStockAction'}
                    ]);

                    GetLog()->info("update $Attributename (ProductLowOnStockAction=1) for language $LanguageCode");

                    # add new text to ProductLongerDeliveryTimeText for ProductLowOnStockAction = 1
                    $sql = <<'END_SQL';
UPDATE localizedstringattribute
SET value = CONCAT(?, '<br />', value)
WHERE languageid = ?
  AND attributeid = ?
  AND objectid IN (
    SELECT objectid
    FROM integerattribute
    WHERE attributeid = ?
      AND value = 1
  )
END_SQL

                    GetLog()->info("update VALUE= $AttributeValue ,  LID: $LanguageID , $Attributename ID: $AttributeIDs{$Attributename}  , ProductLowOnStockAction ID:$AttributeIDs{'ProductLowOnStockAction'}"); ## no critic qw(ProhibitLongLines)

                    $dbi->do( $sql, [
                        $AttributeValue,
                        $LanguageID,
                        $AttributeIDs{$Attributename},
                        $AttributeIDs{'ProductLowOnStockAction'}
                    ]);

                    GetLog()->info("insert $Attributename for language $LanguageCode");
                    # insert ProductLongerDeliveryTimeText if not exists
                    $sql = <<'END_SQL';
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT shopid, ?, ?, ?
FROM shop
WHERE shopid NOT IN(
  SELECT objectid
  FROM localizedstringattribute
  WHERE languageid = ?
  AND attributeid = ?
)
END_SQL
                    $dbi->do($sql, [
                        $AttributeIDs{$Attributename},
                        $LanguageID,
                        $AttributeValue,
                        $LanguageID,
                        $AttributeIDs{$Attributename}
                    ]);

                } else { # all other attributes are updated

                    GetLog()->info("update $Attributename for language $LanguageCode");
                    # update
                    $sql = <<'END_SQL';
UPDATE localizedstringattribute
SET value=?
WHERE languageid=?
  AND attributeid=?
END_SQL
                    $dbi->do($sql, [$AttributeValue,$LanguageID,$AttributeIDs{$Attributename}]);

                    GetLog()->info("insert $Attributename for language $LanguageCode");
                    # insert if not exists
                    $sql = <<'END_SQL';
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT shopid,?,?,?
FROM shop
WHERE shopid NOT IN(
  SELECT objectid
  FROM localizedstringattribute
  WHERE languageid=?
    AND attributeid=?
)
END_SQL
                    $dbi->do($sql, [
                        $AttributeIDs{$Attributename},
                        $LanguageID,
                        $AttributeValue,
                        $LanguageID,
                        $AttributeIDs{$Attributename}
                    ]);
                }#end if $Attributename eq 'ProductLongerDeliveryTimeText' {
            }# end foreach Attribute

        }# end foreach language

        # set ProductLowOnStockAction to 0 where ProductLowOnStockAction==1
        GetLog()->info('set ProductLowOnStockAction to 0 where ProductLowOnStockAction=1');
            $sql = <<'END_SQL';
UPDATE integerattribute
SET value=0
WHERE attributeid=?
AND value=1
END_SQL
        $dbi->do($sql, [$AttributeIDs{'ProductLowOnStockAction'}]);

    }); # end transaction

    GetLog()->info('end update or insert new availability texts for EPG-4295');

    return;
}

# build perl hash structure from translation.*.xml files
sub XMLToPerl {
    my ($FilePrefix) = @_;

    my @LanguageCodes = map { $_->{Code2} } grep { $_->{'IsUsed'} } @{GetAllInfoLanguages()};
    my $hStructure;
    foreach my $Code2 ( @LanguageCodes  ) {
        foreach my $File (sort <$FilePrefix*.$Code2.xml>) {
            my $hStruct = DE_EPAGES::Dictionary::API::XML::TranslationImportHandler::ImportTranslation($File);
            $hStructure->{$Code2} = $hStruct->{$Code2}->{'.'};
        }
    }
   return $hStructure;
}

#========================================================================================
# §function     patchDBStructure_6_13_4
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_13_4;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version  to version .
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function,  | string
#========================================================================================
sub patchDBStructure_6_13_4 {
    my $self = shift;

    my $dbi = GetCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/product.sql
        GetLog()->info('add column product.iscustomizable');
        unless ($dbi->existsTableColumn('product', 'iscustomizable')) {
            $dbi->addTableColumn('product', 'iscustomizable bool not null');
        }
        $dbi->modifyTableColumnDropDefault('product','iscustomizable');
        GetLog()->info('create index product.i_product_weightunitid');
        $dbi->createIndex('product', 'i_product_weightunitid', undef, ['weightunitid'])
            unless ($dbi->existsIndex('product','i_product_weightunitid'));

        GetLog()->info('create index product.i_product_orderunitid');
        $dbi->createIndex('product', 'i_product_orderunitid', undef, ['orderunitid'])
            unless ($dbi->existsIndex('product','i_product_orderunitid'));

        GetLog()->info('create index product.i_product_defaultsubproductid');
        $dbi->createIndex('product', 'i_product_defaultsubproductid', undef, ['defaultsubproductid'])
            unless ($dbi->existsIndex('product','i_product_defaultsubproductid'));

        GetLog()->info('create index product.i_product_refunitid');
        $dbi->createIndex('product', 'i_product_refunitid', undef, ['refunitid'])
            unless ($dbi->existsIndex('product','i_product_refunitid'));

        GetLog()->info('create index product.i_product_durationunitid');
        $dbi->createIndex('product', 'i_product_durationunitid', undef, ['durationunitid'])
            unless ($dbi->existsIndex('product','i_product_durationunitid'));

        #changes for file /Database/mysql/Tables/selectedvariation.sql
        GetLog()->info('create index selectedvariation.i_selectvariation_attributeid');
        $dbi->createIndex('selectedvariation', 'i_selectvariation_attributeid', undef, ['attributeid'])
            unless ($dbi->existsIndex('selectedvariation','i_selectvariation_attributeid'));

        #changes for file /Database/mysql/Tables/productprice.sql
        GetLog()->info('create index productprice.i_productprice_currency');
        $dbi->createIndex('productprice', 'i_productprice_currency', undef, ['currencyid'])
            unless ($dbi->existsIndex('productprice','i_productprice_currency'));

        #changes for file /Database/mysql/Tables/productprepayment.sql
        GetLog()->info('create index productprepayment.i_productprepayment_currency');
        $dbi->createIndex('productprepayment', 'i_productprepayment_currency', undef, ['currencyid'])
            unless ($dbi->existsIndex('productprepayment','i_productprepayment_currency'));

    }

    $self->importPatchDir('Patch_6_13_4', 'PatchDBStructure');

    GetLog()->info('fill type producttype');
    my $ClassIDProductType = $self->getClassID('ProductType');
    $dbi->do( <<'END_SQL', [$ClassIDProductType]);
        INSERT INTO producttype (classid, customizeisfontsizeselectable, customizefontsize)
        SELECT objectid, 0, 12
        FROM object
        WHERE classid = ?
          AND objectid NOT IN (SELECT classid FROM producttype)
END_SQL

    return ;
}

#========================================================================================
# §function     patch_6_13_4
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patch_6_13_4;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge data in active store (database connection)
#               from version  to version .
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function,  | string
#========================================================================================
sub patch_6_13_4 {
    my $self = shift;

    #dont forget these command, it call data changes from patch.xml file
    $self->importPatchDir('Patch_6_13_4', 'Patch');

    my $dbi = GetCurrentDBHandle();
    if($dbi->driverName() eq 'mysql'){

        ## no critic qw(ProhibitLongLines)

        #EPG-4295
        GetLog()->info( 'EPG-4295 replace attribute texts' );

        my $hStructure = {
            'en' => {
                    'ProductStockAvailableText' => 'In stock#IF(#DeliveryPeriod)<br />can be shipped within #DeliveryPeriod days#ENDIF',
                    'ProductStockCriticalText' =>  'Still in stock#IF(#DeliveryPeriod)<br />can be shipped within #DeliveryPeriod days#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Not in stock',
                    'ProductNoAddToBasketText' => 'Currently unavailable',
                    'ProductNoStockDataText' =>  '#IF(#DeliveryPeriod)<br />can be shipped within #DeliveryPeriod days#ENDIF',
            },
            'de' => {
                    'ProductStockAvailableText' => 'Auf Lager#IF(#DeliveryPeriod)<br />innerhalb #DeliveryPeriod Tagen lieferbar#ENDIF',
                    'ProductStockCriticalText' => 'Noch auf Lager#IF(#DeliveryPeriod)<br />innerhalb #DeliveryPeriod Tagen lieferbar#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Nicht auf Lager',
                    'ProductNoAddToBasketText' => 'Zur Zeit nicht lieferbar',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />innerhalb #DeliveryPeriod Tagen lieferbar#ENDIF',
            },
            'ca' => {
                    'ProductStockAvailableText' => 'En estoc#IF(#DeliveryPeriod)<br />es pot enviar en #DeliveryPeriod dies#ENDIF',
                    'ProductStockCriticalText' => 'Encara en estoc#IF(#DeliveryPeriod)<br />es pot enviar en #DeliveryPeriod dies#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'No hi és en estoc',
                    'ProductNoAddToBasketText' => 'Actualment no està disponible',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />es pot enviar en #DeliveryPeriod dies#ENDIF',
            },
            'cs' => {
                    'ProductStockAvailableText' => 'Na skladě#IF(#DeliveryPeriod)<br />lze dodat během #DeliveryPeriod dnů#ENDIF',
                    'ProductStockCriticalText' => 'Stále na skladě#IF(#DeliveryPeriod)<br />lze dodat během #DeliveryPeriod dnů#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Není na skladě',
                    'ProductNoAddToBasketText' => 'Aktuálně nedostupné',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />je možné odeslat během #DeliveryPeriod dnů#ENDIF',
            },
            'da' => {
                    'ProductStockAvailableText' => 'På lager #IF(#DeliveryPeriod)<br />kan sendes inden for #DeliveryPeriod dage#ENDIF',
                    'ProductStockCriticalText' => 'Stadig på lager#IF(#DeliveryPeriod)<br />kan sendes inden for #DeliveryPeriod dage#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Ikke på lager',
                    'ProductNoAddToBasketText' => 'Ikke på lager i øjeblikket',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />kan sendes inden for dage #DeliveryPeriod dage#ENDIF',
            },
            'es' => {
                    'ProductStockAvailableText' => 'En existencias #IF(#DeliveryPeriod)<br />se puede enviar en #DeliveryPeriod días#ENDIF',
                    'ProductStockCriticalText' => 'Todavía en existencias#IF(#DeliveryPeriod)<br />se puede enviar en #DeliveryPeriod días#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'No hay existencias',
                    'ProductNoAddToBasketText' => 'No disponible en estos momentos',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />se puede enviar en #DeliveryPeriod días#ENDIF',
            },
            'fi' => {
                    'ProductStockAvailableText' => 'Varastossa#IF(#DeliveryPeriod)<br />voidaan lähettää #DeliveryPeriod päivän kuluessa#ENDIF',
                    'ProductStockCriticalText' => 'Yhä varastossa#IF(#DeliveryPeriod)<br />voidaan lähettää #DeliveryPeriod päivän kuluessa#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Ei varastossa',
                    'ProductNoAddToBasketText' => 'Ei saatavilla tällä hetkellä',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />voidaan lähettää #DeliveryPeriod päivän kuluessa#ENDIF',
            },
            'fr' => {
                    'ProductStockAvailableText' => 'En stock#IF(#DeliveryPeriod)<br />Expédition possible sous #DeliveryPeriod jours#ENDIF',
                    'ProductStockCriticalText' => 'Encore en stock#IF(#DeliveryPeriod)<br />Expédition possible sous #DeliveryPeriod jours#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Pas en stock',
                    'ProductNoAddToBasketText' => 'Actuellement indisponible',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />Expédition possible sous #DeliveryPeriod jours#ENDIF',
            },
            'it' => {
                    'ProductStockAvailableText' => 'Disponibile#IF(#DeliveryPeriod)<br />possibilità di spedizione entro #DeliveryPeriod giorni#ENDIF',
                    'ProductStockCriticalText' => 'Ancora disponibile#IF(#DeliveryPeriod)<br />possibilità di spedizione entro #DeliveryPeriod giorni#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Non disponibile',
                    'ProductNoAddToBasketText' => 'Attualmente non disponibile',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />possibilità di spedizione entro #DeliveryPeriod giorni#ENDIF',
            },
            'nl' => {
                    'ProductStockAvailableText' => 'Op voorraad#IF(#DeliveryPeriod)<br />kan verzonden worden binnen #DeliveryPeriod dagen#ENDIF',
                    'ProductStockCriticalText' => 'Nog steeds op voorraad#IF(#DeliveryPeriod)<br />kan verzonden worden binnen #DeliveryPeriod dagen#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Niet op voorraad',
                    'ProductNoAddToBasketText' => 'Momenteel niet beschikbaar',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />kan verzonden worden binnen #DeliveryPeriod dagen#ENDIF',
            },
             'no' => {
                    'ProductStockAvailableText' => 'På lager #IF(#DeliveryPeriod)<br />kan sendes innen #DeliveryPeriod dager#ENDIF',
                    'ProductStockCriticalText' => 'Fremdeles på lager #IF(#DeliveryPeriod)<br />kan sendes innen #DeliveryPeriod dager#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Ikke på lager',
                    'ProductNoAddToBasketText' => 'Ikke tilgjengelig for øyeblikket',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />kan sendes innen #DeliveryPeriod dager#ENDIF',
            },
             'ru' => {
                    'ProductStockAvailableText' => 'В наличии; возможна до\xD1?тавка #IF(#DeliveryPeriod)<br /> в течение #DeliveryPeriod дней#ENDIF',
                    'ProductStockCriticalText' => 'До \xD1?их пор в наличии; возможна до\xD1?тавка #IF(#DeliveryPeriod)<br />в течение #DeliveryPeriod дней#ENDIF',
                    'ProductLongerDeliveryTimeText' => '\xD0?ет в наличии',
                    'ProductNoAddToBasketText' => 'В на\xD1?то\xD1?щее врем\xD1? от\xD1?ут\xD1?твует',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />возможна до\xD1?тавка в течение #DeliveryPeriod дней#ENDIF',
            },
             'sv' => {
                    'ProductStockAvailableText' => 'I lager#IF(#DeliveryPeriod)<br />kan levereras inom #DeliveryPeriod dagar#ENDIF',
                    'ProductStockCriticalText' => 'Fortfarande i lager#IF(#DeliveryPeriod)<br />kan levereras inom #DeliveryPeriod dagar#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Inte i lager',
                    'ProductNoAddToBasketText' => 'För närvarande inte tillgänglig',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />kan levereras inom #DeliveryPeriod dagar#ENDIF',
            },
             'en-US' => {
                    'ProductStockAvailableText' => 'In stock#IF(#DeliveryPeriod)<br />can be shipped within #DeliveryPeriod days#ENDIF',
                    'ProductStockCriticalText' => 'Still in stock#IF(#DeliveryPeriod)<br />can be shipped within #DeliveryPeriod days#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Not in stock',
                    'ProductNoAddToBasketText' => 'Currently unavailable',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />can be shipped within #DeliveryPeriod days#ENDIF',
             },
            'pt' => {
                    'ProductStockAvailableText' => 'Disponível#IF(#DeliveryPeriod)<br />pode ser enviado dentro de #DeliveryPeriod dias#ENDIF',
                    'ProductStockCriticalText' => 'Disponível em quantidade limitada#IF(#DeliveryPeriod)<br />pode ser enviado dentro de #DeliveryPeriod dias#ENDIF',
                    'ProductLongerDeliveryTimeText' => 'Esgotado',
                    'ProductNoAddToBasketText' => 'Actualmente indisponível',
                    'ProductNoStockDataText' => '#IF(#DeliveryPeriod)<br />Disponível em quantidade limitada pode ser enviado dentro de #DeliveryPeriod dias#ENDIF',
            }

        };

        ## use critic

        # import structure for existing and new attributes and update each shop with new texts
        # the text of attribute 'ProductLongerDeliveryTimeText' is dependend from attribute 'ProductLowOnStockAction'
        $self->patchStockLevelTexts_6_13_4($hStructure);
    }

    #return the new version number, saved in db
    return ;
}

#========================================================================================
# §function     patch_6_14_0
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patch_6_14_0;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge data in active store (database connection)
#               from version  to version .
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function,  | string
#========================================================================================
sub patch_6_14_0 {
    my $self = shift;

    #dont forget these command, it call data changes from patch.xml file
    $self->importPatchDir('Patch_6_14_0', 'Patch');

    my $dbi = GetCurrentDBHandle();

    GetLog()->info( 'EPG-23156: delete SEOQuality value of variation products' );
    my $SEOQualityAttributeID = LoadClassByAlias('ContentObject')->attribute('SEOQuality')->id;
    $dbi->do( <<END_SQL, [ $SEOQualityAttributeID ] );
        DELETE ia
        FROM integerattribute ia
        JOIN product p ON p.productid = ia.objectid
        WHERE ia.attributeid = ? AND p.superproductid IS NOT NULL
END_SQL

    GetLog()->info( 'EPG-23412: set the layout of existing product types to the old default layout' );
    my %OldDefaults = (
        'Content-Base' => 'ImageLeft',
        'Content-Variation' => 'VariationSelectBox',
        'Content-Base-SelectVariation' => 'SelectBox',
        'Content-Bundles' => 'BundlesCompactList',
    );
    my $PageType = LoadPageTypeByAlias('SF-Product');
    foreach my $TemplateName (keys %OldDefaults) {
        my $Template = $PageType->getTemplate( $TemplateName );
        my $TemplateType = $Template->getTemplateType( $OldDefaults{$TemplateName} );

        $dbi->do( <<END_SQL, [ $Template->id, $TemplateType->id, $Template->id ] );
            INSERT INTO templatemap ( objectid, templateid, templatetypeid )
            SELECT classid, ?, ?
            FROM producttype
            WHERE classid NOT IN(
                SELECT objectid
                FROM templatemap
                WHERE templateid = ?
            )
END_SQL
    }

    return ;
}

#========================================================================================
# §function     patchDBStructure_6_14_2
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_14_2;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version  to version .
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function,  | string
#========================================================================================
sub patchDBStructure_6_14_2 {
    my $self = shift;

    #changes for file /Database/mysql/Tables/product.sql
    my @aIndexes = qw(
        fk_product_product_1
        fk_product_unitofmeasurement
        fk_product_unitofmeasuremen_1
        fk_product_unitofmeasuremen_2
        fk_product_unitofmeasuremen_3
    );
    $self->dropIndexesIfExist( 'product', \@aIndexes );

    #changes for file /Database/mysql/Tables/productprepayment.sql
    $self->dropIndexesIfExist( 'productprepayment', [ 'fk_productprepayment_currency' ] );

    #changes for file /Database/mysql/Tables/productprice.sql
    $self->dropIndexesIfExist( 'productprice', [ 'fk_productprice_currency' ] );

    #changes for file /Database/mysql/Tables/selectedvariation.sql
    $self->dropIndexesIfExist( 'selectedvariation', [ 'fk_selectedvariation_attribute' ] );

    return ;
}

#========================================================================================
# §function     patchDBStructure_6_14_4
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_14_4;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version  to version .
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function,  | string
#========================================================================================
sub patchDBStructure_6_14_4 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    $self->importPatchDir('Patch_6_14_4', 'PatchDBStructure');

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/product.sql
        $self->logInfo('create index product.i_product_ecoparticipationcategory');
        $dbi->createIndex('product', 'i_product_ecoparticipationcategory', undef, ['ecoparticipationcategoryid'])
            unless ($dbi->existsIndex('product','i_product_ecoparticipationcategory'));

        $self->logInfo('drop indices \'i_product_ecopartcategory\' from table \'product\'');
        $dbi->dropIndexesIfExist('product', ['i_product_ecopartcategory']);

        #ok
        if ($dbi->existsTableColumn('product', 'variationstring')) {
            $self->logInfo('drop column product.variationstring');
            $dbi->dropTableColumn('product','variationstring');
        }

        unless ($dbi->existsTableColumn('product', 'manufacturerid')) {
            $self->logInfo('add column product.manufacturerid');
            $dbi->addTableColumnAfterColumn('product', 'stocklevelalert', 'manufacturerid integer null');
        }
        $self->logInfo('recreate index product.i_product_super');
        $dbi->recreateIndex('product', 'i_product_super', undef,
            ['superproductid'],
            { 'superproductid' => 'ASC' }
        );

        $self->logInfo('create index product.i_product_manufacturerid');
        $dbi->createIndex('product', 'i_product_manufacturerid', undef, ['manufacturerid'])
            unless ($dbi->existsIndex('product','i_product_manufacturerid'));

        if (not $dbi->existsForeignKey('fk_product_manufacturerid')) {
            $self->logInfo('add foreign key product.fk_product_manufacturerid');
            $dbi->addForeignKey(
                'product',
                'fk_product_manufacturerid',
                [
                    'manufacturerid',
                ],
                'productmanufacturer',
                [
                    'manufacturerid',
                ]
            );
        }
    }

    return ;
}

#========================================================================================
# §function     patch_6_14_4
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patch_6_14_4;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge in active store (database connection)
#               from version  to version .
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function,  | string
#========================================================================================
sub patch_6_14_4 {
    my $self = shift;

    my $dbi = GetCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        # start transaction for all attribute changes
        GetLog()->info('start transaction for changes on tables \'product\' and \'productmanufacturer\'');

        if ($dbi->existsTableColumn('product', 'manufacturer')) {

            Transaction ( sub {

                GetLog()->info('insert manufacturer names into \'productmanufacturer\' from \'product.manufacturer\'');
                $dbi->do( <<END_SQL, [] );
                    INSERT INTO productmanufacturer(siteid,name)
                    SELECT DISTINCT shopid, manufacturer FROM product
                    WHERE manufacturer IS NOT NULL;
END_SQL

                GetLog()->info('update \'product.manufacturerid\' from \'productmanufacturer.manufacturerid\'');
                $dbi->do( <<END_SQL, [] );
                    UPDATE product p
                    INNER JOIN productmanufacturer pm ON p.manufacturer = pm.name AND p.shopid = pm.siteid
                    SET p.manufacturerid =  pm.manufacturerid
END_SQL

                GetLog()->info('dropTableColumn \'manufacturer\' on table \'product\'');
                if ($dbi->existsTableColumn('product', 'manufacturer')) {
                    GetLog()->info('drop column product.manufacturer');
                    $dbi->dropTableColumn('product','manufacturer');
                }

            }); #end transaction

        }# end if exists

    }

    GetLog()->info( 'import patch dir Patch_6_14_4' );
    $self->importPatchDir('Patch_6_14_4', 'Patch');

    #return the new version number, saved in db
    return ;
}

#========================================================================================
# §function     patchDBStructure_6_15_2
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_15_2;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '6.15.2' to version '6.15.3'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.15.3' | string
#========================================================================================
sub patchDBStructure_6_15_2 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){
        #changes for file /Database/mysql/Tables/fileoption.sql
        $self->logInfo('drop indices \'i_optionid\' from table \'fileoption\'');
        $dbi->dropIndexesIfExist('fileoption', ['i_optionid']);

        #changes for file /Database/mysql/Tables/textoption.sql
        $self->logInfo('drop indices \'i_optionid\' from table \'textoption\'');
        $dbi->dropIndexesIfExist('textoption', ['i_optionid']);
    }

    return '6.15.3';
}

#========================================================================================
# §function     patch_6_15_3
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patch_6_15_3;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge in active store (database connection)
#                from version '6.15.3' to version '6.15.4'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.15.4' | string
#========================================================================================
sub patch_6_15_3 {
    my $self = shift;

    GetLog()->info('import patch dir Patch_6_15_3');
    $self->importPatchDir('Patch_6_15_3', 'Patch');

    my $dbi = GetCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        # cf. EPG-27975 (Product/Scripts/insertMissingDefaultVariationValues.pl)
        GetLog()->info('start transaction for changes on tables \'selectpredefattribute\' and \'selectedvariations\'');
        Transaction (sub {
            GetLog()->info('receive all subproducts and the variation values which need to be updated');
            ## no critic qw(ProhibitLongLines)
            my $aaRows = $dbi->execute(<<END_SQL, []);
                SELECT p.productid, spda1.attributeid, spda1.predefattributeid
                FROM product p
                INNER JOIN selectpredefattribute spda1 ON (p.superproductid = spda1.objectid)
                LEFT OUTER JOIN selectpredefattribute spda2 ON (p.productid = spda2.objectid and spda1.attributeid = spda2.attributeid)
                WHERE spda2.objectid is null
                    AND (p.superproductid, spda1.attributeid) IN (
                        SELECT productid, attributeid
                        FROM selectedvariation
                    )
END_SQL

            ## use critic
            GetLog()->info('update subproducts');
            foreach my $aRow (@$aaRows) {
                my ($ProductID, $AttributeID, $PreDefAttributeID) = @{$aRow};

                my $Product = LoadObject($ProductID);
                my $Attribute = LoadObject($AttributeID);
                my $PreDefAttribute = LoadObject($PreDefAttributeID);

                # (1) update variation
                $Product->set({ $Attribute->alias => $PreDefAttribute });
                # (2) update master product
                my $SuperProduct = $Product->get('SuperProduct');
                $SuperProduct->set({ $Attribute->alias => $PreDefAttribute });
                my $aSelectedVariations = $SuperProduct->get('SelectedVariations');
                push @$aSelectedVariations, {'SelectedID' => $PreDefAttributeID};
                $SuperProduct->set({ 'SelectedVariations' => $aSelectedVariations });
            }
        });
    }

    return '6.15.4';
}

#========================================================================================
# §function     patch_6_15_4
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patch_6_15_4;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge in active store (database connection)
#                from version '6.15.4' to version '6.16.0'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.16.0' | string
#========================================================================================
sub patch_6_15_4 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    $self->importPatchDir('Patch_6_15_4', 'Patch');

    $self->logInfo('shift old product layout to the new ones');

    # layouts (template types) are located at SF-Product/Content-Base
    my $PageType = LoadPageTypeByAlias('SF-Product');
    my $TemplateTypeFolder = $PageType->templateTypeFolder('Content-Base');

    # map the old template types to their corresponding new equivalent (add "_New" to the alias)
    # create a hash of schema ( old id => corresponding new id )
    my %OldNewMapping = ();
    foreach my $TemplateType (@{$TemplateTypeFolder->children}) {
        # skip the new template types here
        next if $TemplateType->alias =~ m/_New/;

        %OldNewMapping = (%OldNewMapping,
            $TemplateType->id => $TemplateTypeFolder->child($TemplateType->alias .'_New')->id
        );
    }

    # mark affected shops
    my $Class = LoadClassByAlias('Shop');
    my $AttributeID = $Class->attribute('InitialMsgSwitchedProductLayouts')->id;

    $self->logInfo('mark affected shops at booleanattribute');
    foreach my $OldTemplateTypeID (keys %OldNewMapping) {
        $dbi->do(<<'END_SQL', [ $AttributeID, $OldTemplateTypeID, $AttributeID ]);
INSERT INTO booleanattribute (objectid, attributeid, value)
SELECT DISTINCT o.siteid, ?, 1
FROM templatemap tm INNER JOIN object o ON o.objectid = tm.objectid
WHERE tm.templatetypeid = ?
AND o.siteid NOT IN (
    SELECT objectid
    FROM booleanattribute
    WHERE attributeid = ?
    AND value = 1
)
END_SQL
    }

    # replace the old template type id's with the id's of the new ones
    $self->logInfo('replace the old template type ids with the ids of the new ones at templatemap');
    while (my ($OldID, $NewID) = each %OldNewMapping) {
        $dbi->do(<<'END_SQL', [ $NewID, $OldID ]);
UPDATE templatemap
SET templatetypeid = ?
WHERE templatetypeid = ?
END_SQL
    }

    return '6.16.0';
}

#========================================================================================
# §function     patchDBStructure_6_15_4
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_15_4;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '6.15.4' to version '6.16.0'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.16.0' | string
#========================================================================================
sub patchDBStructure_6_15_4 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/lineitemproduct.sql
        if ($dbi->existsForeignKey('fk_lineitemproduct_product')) {
            $self->logInfo('drop foreign key lineitemproduct.fk_lineitemproduct_product');
            $dbi->dropForeignKey('lineitemproduct', 'fk_lineitemproduct_product');
        }
        if ($dbi->existsForeignKey('fk_lineitemproduct_orderunit')) {
            $self->logInfo('drop foreign key lineitemproduct.fk_lineitemproduct_orderunit');
            $dbi->dropForeignKey('lineitemproduct', 'fk_lineitemproduct_orderunit');
        }
        if ($dbi->existsForeignKey('fk_lineitemproduct_lineitem')) {
            $self->logInfo('drop foreign key lineitemproduct.fk_lineitemproduct_lineitem');
            $dbi->dropForeignKey('lineitemproduct', 'fk_lineitemproduct_lineitem');
        }
        if ($dbi->existsForeignKey('fk_lineitemproduct_weightunit')) {
            $self->logInfo('drop foreign key lineitemproduct.fk_lineitemproduct_weightunit');
            $dbi->dropForeignKey('lineitemproduct', 'fk_lineitemproduct_weightunit');
        }
        if (not $dbi->existsForeignKey('fk_lineitemproduct_orderunitid')) {
            $self->logInfo('add foreign key lineitemproduct.fk_lineitemproduct_orderunitid');
            $dbi->addForeignKey(
                'lineitemproduct',
                'fk_lineitemproduct_orderunitid',
                [
                    'orderunitid',
                ],
                'unitofmeasurement',
                [
                    'unitid',
                ]
            );
        }
        if (not $dbi->existsForeignKey('fk_lineitemproduct_weightunitid')) {
            $self->logInfo('add foreign key lineitemproduct.fk_lineitemproduct_weightunitid');
            $dbi->addForeignKey(
                'lineitemproduct',
                'fk_lineitemproduct_weightunitid',
                [
                    'weightunitid',
                ],
                'unitofmeasurement',
                [
                    'unitid',
                ]
            );
        }
        if (not $dbi->existsForeignKey('fk_lineitemproduct_lineitemproductid')) {
            $self->logInfo('add foreign key lineitemproduct.fk_lineitemproduct_lineitemproductid');
            $dbi->addForeignKey(
                'lineitemproduct',
                'fk_lineitemproduct_lineitemproductid',
                [
                    'lineitemproductid',
                ],
                'lineitem',
                [
                    'lineitemid',
                ]
            );
        }
        if (not $dbi->existsForeignKey('fk_lineitemproduct_productid')) {
            $self->logInfo('add foreign key lineitemproduct.fk_lineitemproduct_productid');
            $dbi->addForeignKey(
                'lineitemproduct',
                'fk_lineitemproduct_productid',
                [
                    'productid',
                ],
                'product',
                [
                    'productid',
                ]
            );
        }

        #changes for file /Database/mysql/Tables/selectedvariation.sql
        if ($dbi->existsForeignKey('fk_selectedvariation_predefatt')) {
            $self->logInfo('drop foreign key selectedvariation.fk_selectedvariation_predefatt');
            $dbi->dropForeignKey('selectedvariation', 'fk_selectedvariation_predefatt');
        }
        if ($dbi->existsForeignKey('fk_selectedvariation_product')) {
            $self->logInfo('drop foreign key selectedvariation.fk_selectedvariation_product');
            $dbi->dropForeignKey('selectedvariation', 'fk_selectedvariation_product');
        }
        if ($dbi->existsForeignKey('fk_selectedvariation_attribute')) {
            $self->logInfo('drop foreign key selectedvariation.fk_selectedvariation_attribute');
            $dbi->dropForeignKey('selectedvariation', 'fk_selectedvariation_attribute');
        }
        if (not $dbi->existsForeignKey('fk_selectedvariation_predefattributeid')) {
            $self->logInfo('add foreign key selectedvariation.fk_selectedvariation_predefattributeid');
            $dbi->addForeignKey(
                'selectedvariation',
                'fk_selectedvariation_predefattributeid',
                [
                    'predefattributeid',
                ],
                'predefattribute',
                [
                    'predefattributeid',
                ]
            );
        }
        if (not $dbi->existsForeignKey('fk_selectedvariation_attributeid')) {
            $self->logInfo('add foreign key selectedvariation.fk_selectedvariation_attributeid');
            $dbi->addForeignKey(
                'selectedvariation',
                'fk_selectedvariation_attributeid',
                [
                    'attributeid',
                ],
                'attribute',
                [
                    'attributeid',
                ]
            );
        }
        if (not $dbi->existsForeignKey('fk_selectedvariation_productid')) {
            $self->logInfo('add foreign key selectedvariation.fk_selectedvariation_productid');
            $dbi->addForeignKey(
                'selectedvariation',
                'fk_selectedvariation_productid',
                [
                    'productid',
                ],
                'product',
                [
                    'productid',
                ]
            );
        }

        #changes for file /Database/mysql/Tables/downloadproductmap.sql
        if ($dbi->existsForeignKey('fk_downloadproductmap_product')) {
            $self->logInfo('drop foreign key downloadproductmap.fk_downloadproductmap_product');
            $dbi->dropForeignKey('downloadproductmap', 'fk_downloadproductmap_product');
        }
        if (not $dbi->existsForeignKey('fk_downloadproductmap_downloadproductid')) {
            $self->logInfo('add foreign key downloadproductmap.fk_downloadproductmap_downloadproductid');
            $dbi->addForeignKey(
                'downloadproductmap',
                'fk_downloadproductmap_downloadproductid',
                [
                    'downloadproductid',
                ],
                'product',
                [
                    'productid',
                ]
            );
        }

        #changes for file /Database/mysql/Tables/product.sql
        if ($dbi->existsForeignKey('fk_product_contentobject')) {
            $self->logInfo('drop foreign key product.fk_product_contentobject');
            $dbi->dropForeignKey('product', 'fk_product_contentobject');
        }
        unless ($dbi->existsTableColumn('product', 'imagemediumsmall')) {
            $self->logInfo('add column product.imagemediumsmall');
            $dbi->addTableColumnAfterColumn('product', 'availabilitydate', 'imagemediumsmall varchar(255) null');
        }
        unless ($dbi->existsTableColumn('product', 'imagemediumlarge')) {
            $self->logInfo('add column product.imagemediumlarge');
            $dbi->addTableColumnAfterColumn('product', 'imagemediumsmall', 'imagemediumlarge varchar(255) null');
        }
        unless ($dbi->existsTableColumn('product', 'expirydate')) {
            $self->logInfo('add column product.expirydate');
            $dbi->addTableColumnAfterColumn('product', 'iscustomizable', 'expirydate datetime null');
        }
        if (not $dbi->existsForeignKey('fk_product_productid')) {
            $self->logInfo('add foreign key product.fk_product_productid');
            $dbi->addForeignKey(
                'product',
                'fk_product_productid',
                [
                    'productid',
                ],
                'contentobject',
                [
                    'contentobjectid',
                ]
            );
        }

        #changes for file /Database/mysql/Tables/lineitemproductmaxdownload.sql
        if ($dbi->existsForeignKey('fk_lineitemmaxdownload_downloadproductmap')) {
            $self->logInfo('drop foreign key lineitemproductmaxdownload.fk_lineitemmaxdownload_downloadproductmap');
            $dbi->dropForeignKey('lineitemproductmaxdownload', 'fk_lineitemmaxdownload_downloadproductmap');
        }
        if (not $dbi->existsForeignKey('fk_lineitemproductmaxdownload_downloadproductmapid')) {
            $self->logInfo('add foreign key lineitemproductmaxdownload.fk_lineitemproductmaxdownload_downloadproductmapid');
            $dbi->addForeignKey(
                'lineitemproductmaxdownload',
                'fk_lineitemproductmaxdownload_downloadproductmapid',
                [
                    'downloadproductmapid',
                ],
                'downloadproductmap',
                [
                    'downloadproductmapid',
                ]
            );
        }

        #changes for file /Database/mysql/Tables/bundleproductmap.sql
        if ($dbi->existsForeignKey('fk_bundleproductmap_product')) {
            $self->logInfo('drop foreign key bundleproductmap.fk_bundleproductmap_product');
            $dbi->dropForeignKey('bundleproductmap', 'fk_bundleproductmap_product');
        }
        if ($dbi->existsForeignKey('fk_bundleproductmap_product_1')) {
            $self->logInfo('drop foreign key bundleproductmap.fk_bundleproductmap_product_1');
            $dbi->dropForeignKey('bundleproductmap', 'fk_bundleproductmap_product_1');
        }
        if (not $dbi->existsForeignKey('fk_bundleproductmap_bundleproductid')) {
            $self->logInfo('add foreign key bundleproductmap.fk_bundleproductmap_bundleproductid');
            $dbi->addForeignKey(
                'bundleproductmap',
                'fk_bundleproductmap_bundleproductid',
                [
                    'bundleproductid',
                ],
                'product',
                [
                    'productid',
                ]
            );
        }
        if (not $dbi->existsForeignKey('fk_bundleproductmap_bundledproductid')) {
            $self->logInfo('add foreign key bundleproductmap.fk_bundleproductmap_bundledproductid');
            $dbi->addForeignKey(
                'bundleproductmap',
                'fk_bundleproductmap_bundledproductid',
                [
                    'bundledproductid',
                ],
                'product',
                [
                    'productid',
                ]
            );
        }

        #changes for file /Database/mysql/Tables/productshippingmethod.sql
        if ($dbi->existsForeignKey('fk_productshippingmethod_produ')) {
            $self->logInfo('drop foreign key productshippingmethod.fk_productshippingmethod_produ');
            $dbi->dropForeignKey('productshippingmethod', 'fk_productshippingmethod_produ');
        }
        if (not $dbi->existsForeignKey('fk_productshippingmethod_productid')) {
            $self->logInfo('add foreign key productshippingmethod.fk_productshippingmethod_productid');
            $dbi->addForeignKey(
                'productshippingmethod',
                'fk_productshippingmethod_productid',
                [
                    'productid',
                ],
                'product',
                [
                    'productid',
                ]
            );
        }

    }

    return '6.16.0';
}

#========================================================================================
# §function     patchDBStructure_6_17_3
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_17_3;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.3' to version '6.17.4'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.17.4' | string
#========================================================================================
sub patchDBStructure_6_17_3 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/lineitemproductmaxdownload.sql
        $self->logInfo('change type of column lineitemproductmaxdownload.counter');
        $dbi->modifyTableColumn('lineitemproductmaxdownload', 'counter integer null');

        unless ($dbi->existsTableColumn('lineitemproductmaxdownload', 'linkguid')) {
            $self->logInfo('add column lineitemproductmaxdownload.linkguid');
            $dbi->addTableColumnAfterColumn('lineitemproductmaxdownload', 'counter', 'linkguid char(36) null');
        }
        $self->logInfo('create index lineitemproductmaxdownload.i_linkguid');
        $dbi->createIndex('lineitemproductmaxdownload', 'i_linkguid', 'UNIQUE ', ['linkguid'])
            unless ($dbi->existsIndex('lineitemproductmaxdownload','i_linkguid'));

    }

    return '6.17.4';
}

#========================================================================================
# §function     patch_6_17_4
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patch_6_17_4;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge in active store (database connection)
#                from version '6.17.4' to version '6.17.5'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.17.5' | string
#========================================================================================
sub patch_6_17_4 {
    my $self = shift;

    $self->importPatchDir('Patch_6_17_4', 'Patch');

    $self->logInfo('populate new table column product.ismaster');
    $self->getCurrentDBHandle->do(<<'END_SQL');
UPDATE product
SET ismaster=1
WHERE productid IN (
  SELECT productid FROM selectedvariation
)
END_SQL

    return '6.17.5';
}

#========================================================================================
# §function     patch_6_17_9
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patch_6_17_9;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge in active store (database connection)
#                from version '6.17.9' to version '6.17.10'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.17.10' | string
#========================================================================================
sub patch_6_17_9 {
    my $self = shift;

    $self->importPatchDir('Patch_6_17_9', 'Patch');

    my $dbi = $self->getCurrentDBHandle();

    $self->logInfo('EPG-36120 fix ismaster for products with no subproducts and active selected variations.');
    $self->logInfo('ismaster is set to 0 and selected variations are deleted for this products');

    $self->logInfo('patch 6.17.9 - create temporary table tmp_clause1');
    $dbi->do( <<END_SQL); # don't use a temporary table, because it may lead to data loss
CREATE TABLE tmp_clause1 (
    productid INT PRIMARY KEY
)
END_SQL

    $self->logInfo('patch 6.17.9 - create temporary table tmp_ismaster');
    $dbi->do( <<END_SQL); # don't use a temporary table, because it may lead to data loss
CREATE TABLE tmp_ismaster (
    productid INT PRIMARY KEY
)
END_SQL

    $self->logInfo('patch 6.17.9 - fill temporary table tmp_clause1 from product');
    $dbi->do( <<END_SQL);
INSERT INTO tmp_clause1(productid)
SELECT distinct superproductid FROM product
WHERE superproductid IS NOT NULL
END_SQL

    $self->logInfo('patch 6.17.9 - fill temporary table tmp_ismaster from product and selectedvariation');
    $dbi->do( <<END_SQL);
INSERT INTO tmp_ismaster(productid)
SELECT productid FROM product
WHERE productid NOT IN( select productid from tmp_clause1)
AND productid IN (SELECT productid FROM selectedvariation)
END_SQL

    $self->logInfo('patch 6.17.9 - fill table product with data from tmp_ismaster');
    $dbi->do( <<END_SQL);
UPDATE product
SET ismaster=0
WHERE productid IN(SELECT productid FROM tmp_ismaster)
END_SQL

    $self->logInfo('patch 6.17.9 - delete some entries in selectedvariation with data from tmp_ismaster');
    $dbi->do( <<END_SQL);
DELETE FROM selectedvariation
WHERE productid IN(SELECT productid FROM tmp_ismaster);
END_SQL

    #delete temporary help tables
    $self->logInfo('patch 6.17.9 - drop temporary table tmp_scaleprice');
    $dbi->dropTable('tmp_clause1');
    $self->logInfo('patch 6.17.9 - drop temporary table tmp_scaleprice');
    $dbi->dropTable('tmp_ismaster');

    return '6.17.10';
}

#========================================================================================
# §function     patchDBStructure_6_17_4
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_17_4;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.4' to version '6.17.5'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.17.5' | string
#========================================================================================
sub patchDBStructure_6_17_4 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/product.sql
        unless ($dbi->existsTableColumn('product', 'ismaster')) {
            $self->logInfo('add column product.ismaster after column product.expirydate');
            $dbi->addTableColumnAfterColumn('product', 'expirydate', 'ismaster bool not null default 0');
            $dbi->modifyTableColumnDropDefault('product','ismaster');
        }

    }

    return '6.17.5';
}

#========================================================================================
# §function     patchDBStructure_6_17_11
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_17_11;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.11' to version '6.17.12'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.17.12' | string
#========================================================================================
sub patchDBStructure_6_17_11 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/product.sql
        unless ($dbi->existsTableColumn('product', 'taxclassdigitalid')) {
            $self->logInfo('add column product.taxclassdigitalid');
            $dbi->addTableColumnAfterColumn('product', 'ismaster', 'taxclassdigitalid integer null');
        }
        $self->logInfo('create index product.i_product_taxclassdigital');
        $dbi->createIndex('product', 'i_product_taxclassdigital', undef, ['taxclassdigitalid'])
            unless ($dbi->existsIndex('product','i_product_taxclassdigital'));

        if (not $dbi->existsForeignKey('fk_product_taxclassdigitalid')) {
            $self->logInfo('add foreign key product.fk_product_taxclassdigitalid');
            $dbi->addForeignKey(
                'product',
                'fk_product_taxclassdigitalid',
                [
                    'taxclassdigitalid',
                ],
                'taxclassdigital',
                [
                    'taxclassdigitalid',
                ]
            );
        }

    }

    return '6.17.12';
}

#========================================================================================
# §function     setStocklevelColumnToDecimal_7_11_0
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Installer->setStocklevelColumnToDecimal_7_11_0();
#----------------------------------------------------------------------------------------
# §description  Changes the data type of column stocklevel and stocklevelalert
#               to decimal(19,4).
#========================================================================================
sub setStocklevelColumnToDecimal_7_11_0 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    return if ($dbi->driverName() ne 'mysql');

    $dbi->do(<<END_SQL);
ALTER TABLE product MODIFY COLUMN stocklevel decimal(19,4),MODIFY COLUMN stocklevelalert decimal(19,4);
END_SQL

    return;
}

#========================================================================================
# §function     patchDBStructure_7_11_0
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_7_11_0;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '7.11.0' to version '7.12.0'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '7.12.0' | string
#========================================================================================
sub patchDBStructure_7_11_0 {
    my $self = shift;

    $self->setStocklevelColumnToDecimal_7_11_0();

    return '7.12.0';
}

1;
