#========================================================================================
# package      DE_EPAGES::ProductPortal::API::Object::PortalSite
# state        public
# base         DE_EPAGES::Object::API::Object::SiteLocale
# description  object interface for PortalSite.
#========================================================================================
package DE_EPAGES::ProductPortal::API::Object::PortalSite;
use base DE_EPAGES::Object::API::Object::SiteLocale;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( Compare GetCurrentDateTime isValid );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Shop::API::Constants qw (Text2TaxModel TaxModel2Text);

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $PortalSite->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'PortalSite' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $PortalSite->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PortalSite = DE_EPAGES::ProductPortal::API::Object::PortalSite->insert($hInfo);
# example      my $Class = LoadClassByAlias('PortalSite');
#               my $PortalSite = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertPortalSite' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $PortalSite | new object | object
# hook         OBJ_InsertPortalSite | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PortalSiteID  - portal site id       - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertPortalSite', {'Object' => $self, 'PortalSiteID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Portal RegisterURL AdminURL PortalURL TaxModel FTPServer FTPPort ExportsShippingPrice EMail Phone ReviewURL LocalName ExportsDownloadProducts IgnoreLastMerchantLogin DeprecationDate); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PortalSite->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeletePortalSite' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeletePortalSite | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PortalSiteID  - portal site id       - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeletePortalSite', {'Object' => $self, 'PortalSiteID' => $self->id });
    LoadClassByAlias('PortalConfig')->deleteObjects('ByPortalSiteID', [$self->id]);
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PortalSite->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdatePortalSite' and 'OBJ_AfterUpdatePortalSite'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdatePortalSite | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PortalSiteID  - portal site id       - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdatePortalSite | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PortalSiteID  - portal site id       - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdatePortalSite', {'Object'=>$self, 'Values'=> $hValues, 'PortalSiteID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdatePortalSite',  {'Object'=>$self, 'Values'=> $hValues, 'PortalSiteID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     exportXMLAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes bit attributes with value 0.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function exportableAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    $hAttributes = $self->SUPER::exportXMLAttributes($Driver, $hAttributes);
    $hAttributes->{'TaxModel'} = TaxModel2Text($hAttributes->{'TaxModel'}) if exists $hAttributes->{'TaxModel'};
    return $hAttributes;
}

#========================================================================================
# function     importXML
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = __PACKAGE__->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Imports an object from an XML file. This function is called at the XML
#               start tag. The end tag triggers the <function importXMLEnd> function.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | XML element structure, see XML::SAX | ref.hash
# input        $Class | class object; the new object will be an instance of this class | object
# input        $hAttributes | attributes hash | ref.hash
# return       $Object | the imported object | object
#========================================================================================
sub importXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    # map TaxModel name to Model id
    if (exists $hAttributes->{'TaxModel'}) {
        $hAttributes->{'TaxModel'} = Text2TaxModel($hAttributes->{'TaxModel'});
    }
    if (!$hAttributes->{'Portal'}) {
        my $Portal = $Handler->topID('Object');
        $hAttributes->{'Portal'} = $Portal;
    }
    my $Object = bless __PACKAGE__->SUPER::importXML($Handler, $hElement, $Class, $hAttributes ), $class;
    return $Object;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::ProductPortal::API::Object::PortalSite->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'ExportsShippingPrice' => 0,
        'ExportsDownloadProducts' => 0,
        'IgnoreLastMerchantLogin' => 0,
    };
}

#========================================================================================
# function     isDeprecated
# state        public
#----------------------------------------------------------------------------------------
# syntax       isDeprecated( );
#----------------------------------------------------------------------------------------
# description  returns if this portal site has been deprecated or not
#----------------------------------------------------------------------------------------
# return       $IsDeprecated | true if portal site holds a deprecation date in the past | boolean
#========================================================================================
sub isDeprecated {
    my $self = shift;

    my $Now = GetCurrentDateTime();
    my $DeprecationDate = $self->get('DeprecationDate');

    return 0 unless isValid($DeprecationDate);
    return 0 if( Compare('DateTime', $Now, $DeprecationDate) < 0 );
    return 1;
}
1;
