#========================================================================================
# package      DE_EPAGES::SEO::API::SEO
# state        public
#----------------------------------------------------------------------------------------
# description  API functions for SEO check on content objects
#========================================================================================
package DE_EPAGES::SEO::API::SEO;
use base qw( Exporter );
use strict;

our @EXPORT_OK = qw (
    GetSEOCheckValues
    CalculateSEOQuality
    GetSEOQualityClass
    AddCategoryImageSEOCheckValues
    AddCategoryDepthSEOCheckValues
    GetShopSEOQualityValues
    CalculateShopSEOQuality
    GetRatingSign
    PerformSEOCheck
    PerformStartPageSEOCheck
    PerformSEOCheckAndUpdateObject
    UpdateSEOQuality
    CheckForBadSEOHTML
);

use DE_EPAGES::Core::API::File qw ( RelativePath );
use DE_EPAGES::Core::API::IDN qw (EncodeDomain);
use DE_EPAGES::Core::API::String qw ( CompareStrings GetWordCount );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);

#========================================================================================
# function     GetSEOCheckValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = GetSEOCheckValues(Shop, $Object, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Calculates various SEO relevant ratings and values for ContentObject
#               SEO checks. Values specific to derived classes can be added/changed
#               unsing a hook.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $Object | content object | object
# input        $LanguageID | language id for localized attributes | integer
# return       $hValues | values and ratings for SEO check, can be extended via hooks
#               <ul>
#                   <li>Name - object name - string</li>
#                   <li>
#                       <ul>Title
#                           <li>Value - text of the title tag - string</li>
#                           <li>Rating - SEO rating for the title tag - integer</li>
#                           <li>RatingClass - CSS class name depending on Rating - string</li>
#                           <li>Characters - Number of characters in the title tag - integer</li>
#                           <li>Words - Number of words in the title tag - integer</li>
#                           <li>Percentage - weighting for the SEO rating for the title tag - float</li>
#                       </ul>
#                   </li>
#                   <li><ul>Url ... </ul> - canonical URL data (similar to Title)</li>
#                   <li><ul>Description ... </ul> - description meta tag data (similar to Title)</li>
#                   <li><ul>AdvDescription ... </ul> - long description data (similar to Title)</li>
#                   <li>... (more values can be added via hook SEOCheckLocalizedTests)</li>
#               </ul>
#               | ref.hash
#========================================================================================
sub GetSEOCheckValues {
    my ($Shop, $Object, $LanguageID) = @_;

    my $hValues;
    my $ShopDomain = $Shop->get('DomainName');
    my $ShopWebUrl = $Shop->get('WebUrl');

    my $ObjectName = $Object->get('Name', $LanguageID);
    $hValues->{'Name'} = $ObjectName;

    my $Title = $Object->get('DisplayTitle', $LanguageID);
    my $ShopName = $Shop->get('Name', $LanguageID);
    my $ShopDisplayTitle = $Shop->get('DisplayTitle', $LanguageID);

    my $TitleRating = _getTitleRating($ObjectName, $Title, $ShopDisplayTitle, $ShopName);
    $hValues->{'Title'} = {
        'Value' => $Title,
        'Rating' => $TitleRating,
        'RatingClass' => qw ( green yellow red red red )[$TitleRating-1],
        'Characters' => length($Title),
        'Words' => GetWordCount($Title)
    };

    my $Canonical = $Object->get('CanonicalURL', $LanguageID);
    my $CanonicalWithoutHttp = $Canonical;
    $CanonicalWithoutHttp =~ s/^https?:\/\///;

    my $UrlRating = _getUrlRating($ShopDomain, $Canonical, $ShopWebUrl);
    $hValues->{'Url'} = {
        'Value' => $Canonical,
        'ValueWithoutHttp' => $CanonicalWithoutHttp,
        'Rating' => $UrlRating,
        'RatingClass' => qw ( green yellow orange red red )[$UrlRating-1],
        'Characters' => defined $Canonical ? length($Canonical) : 0,
    };

    my $Description = $Object->get('DisplayDescription', $LanguageID);
    my $DescriptionRating = _getDescriptionRating($Description);
    $hValues->{'Description'} = {
        'Value' => $Description,
        'Rating' => $DescriptionRating,
        'RatingClass' => qw ( green yellow yellow red red )[$DescriptionRating-1],
        'Characters' => defined $Description ? length($Description) : 0,
        'Words' =>  defined $Description ? GetWordCount($Description) : 0
    };

    $hValues->{'AdvDescription'} = {
        'Value' => $Object->get('Text', $LanguageID)
    };

    TriggerHook('SEOCheckLocalizedTests', {
        'Object'        => $Object,
        'Shop'          => $Shop,
        'Values'        => $hValues,
        'LanguageID'    => $LanguageID,
    });

    my $AdvDescRating = _getAdvDescriptionRating($hValues->{'AdvDescription'}->{'Value'}, $Description);

    $hValues->{'AdvDescription'} = {
        %{$hValues->{'AdvDescription'}},
        'Rating' => $AdvDescRating,
        'RatingClass' => qw ( green yellow orange orange red )[$AdvDescRating-1],
    };
    return $hValues;
}

#========================================================================================
# function     CalculateSEOQuality
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Rating = CalculateSEOQuality($hValues, $Object)
#----------------------------------------------------------------------------------------
# description  Calculates a rating factor which determindes the SEO quality based on the
#               given values in $hValues.
#----------------------------------------------------------------------------------------
# input        $hValues | values and ratings for SEO check | ref.hash
# input        $Object | content object | object
# return       $Rating | SEO quality from 0 .. 100 | integer
#========================================================================================
sub CalculateSEOQuality {
    my ($hValues, $Object) = @_;

    my $hRatingPercentages = $Object->get('SeoCheckRatings');

    my $hRatingAssoc = {
        green   => 1,
        yellow  => 2/3,
        orange  => 1/3,
        red     => 0,
    };

    my $Rating = 0;
    my $PercentageSum = 0;

    foreach my $Key (keys %$hRatingPercentages) {
        my $ItemClass = $hValues->{$Key}->{'RatingClass'};  # red,yellow,etc.
        my $ItemRatingAssoc = $hRatingAssoc->{$ItemClass};  # 1, 2/3, 1/3, 0
        my $Percentage = $hRatingPercentages->{$Key};       # 0.1, 0.25, etc.
        $PercentageSum += $Percentage;
        $Rating += $ItemRatingAssoc * $Percentage;          # 2/3 * 0.15
    }

    # Weighting should be 1 in sum
    # rounding issue workaround
    if(sprintf('%.0f', $PercentageSum*100) != 100) {    ## no critic qw(ProhibitMagicNumbers)
        Warning(
            'Weighting of SEO quality factors should be 100% in sum',
            {
                'PercentageSum' => $PercentageSum,
                'ObjectID' => $Object->id,
            }
        );
    }

    return sprintf('%.0f', $Rating * 100);  ## no critic qw(ProhibitMagicNumbers)
}

#========================================================================================
# function     AddCategoryImageSEOCheckValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       AddCategoryImageSEOCheckValues($Object, $Shop, $hValues)
#----------------------------------------------------------------------------------------
# description  Extends hValues with SEO check for the category image.
#----------------------------------------------------------------------------------------
# input        $Object | content object | object
# input        $Shop | shop object | object
# input        $hValues | values and ratings for SEO check | ref.hash
#========================================================================================
sub AddCategoryImageSEOCheckValues {
    my ($Object, $Shop, $hValues) = @_;

    # image
    my $HasImage = $Object->get('ImageMedium') ? 1 : 0;
    my $ImageRating = $HasImage ? 1 : 2;
    $hValues->{'CategoryImage'} = {
        'Value' => $HasImage,
        'Rating' => $ImageRating,
        'RatingClass' => qw ( green red )[$ImageRating-1],
    };
}

#========================================================================================
# function     AddCategoryDepthSEOCheckValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       AddCategoryDepthSEOCheckValues($Object, $Shop, $hValues)
#----------------------------------------------------------------------------------------
# description  Extends hValues with SEO check for the category hierarchy depth.
#----------------------------------------------------------------------------------------
# input        $Object | content object | object
# input        $Shop | shop object | object
# input        $hValues | values and ratings for SEO check | ref.hash
#========================================================================================
sub AddCategoryDepthSEOCheckValues {
    my ($Object, $Shop, $hValues) = @_;

    # hierarchy depth
    my $CategoryFolder = $Shop->get('Categories');
    my $CategoriesPath = $CategoryFolder->get('Path');          #/Shops/DemoShop/Categories
    my $ObjectPath = $Object->get('Path');               #/Shops/DemoShop/Categories/Equipment/Cleaner
    my $RelPath = RelativePath($CategoriesPath, $ObjectPath);
    my @Parts = split(/\//, $RelPath);
    my $Depth = scalar(@Parts);

    my $DepthRating = _getDepthRating($Depth);
    $hValues->{'CategoryDepth'} = {
        'Value' => $Depth,
        'Rating' => $DepthRating,
        'RatingClass' => qw ( green yellow orange red )[$DepthRating-1],
    };
}

#========================================================================================
# function     GetSEOQualityClass
# state        public
#----------------------------------------------------------------------------------------
# syntax       $QualityClass'} = GetSEOQualityClass($Quality);
#----------------------------------------------------------------------------------------
# description  Returns the CSS class string depending on the SEO quality value.
#----------------------------------------------------------------------------------------
# input        $Quality | SEO quality 0 .. 100 | integer
# return       $Class | css class string | string
#========================================================================================
sub GetSEOQualityClass {
    my ($Quality) = @_;

    my $Class;
    ## no critic qw(ProhibitMagicNumbers)
    if((not defined $Quality) or $Quality < 33) {
        $Class = 'red';
    } elsif($Quality < 66) {
        $Class = 'orange';
    } elsif($Quality < 100) {
        $Class = 'yellow';
    } else {
        $Class = 'green';
    }
    ## use critic
    return $Class;
}

#========================================================================================
# function     GetShopSEOQualityValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = GetShopSEOQualityValues($Shop)
#----------------------------------------------------------------------------------------
# description  Calculates the SEO quality for the shop. Uses general indicators like
#               domain name, page cache, etc.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# return       $hValues | values and ratings for SEO check of the shop, extendable via hooks
#               <ul>
#                   <li>
#                       <ul>DomainName
#                           <li>Rating - SEO rating for the title tag - integer</li>
#                           <li>RatingClass - CSS class name depending on Rating - string</li>
#                       </ul>
#                       ...
#                   </li>
#                   <li>ShopQuality</li>
#                   <li>... </li>
#               </ul>
#               | ref.hash
#========================================================================================
sub GetShopSEOQualityValues {
    my ($Shop) = @_;

    my $hShopAttributes = $Shop->get([qw ( ClosedByMerchant DomainName IsPageCacheUsed SubmittedSitemapDomain SitemapPathFileName ) ]);

    my $DomainNameRating = _getShopDomainNameRating($hShopAttributes->{'DomainName'});
    my $PageCacheRating = $hShopAttributes->{'IsPageCacheUsed'} ? 1 : 2;
    my $Style = $Shop->get('DefaultStyle');
    my $aNavElements = $Style->existsChild('NavBarElements') ? $Style->child('NavBarElements')->children : [];
    my $CopyrightTrickRating = _getShopCopyrightTrickRating($Shop, $aNavElements);
    my $ShopLogoRating = _getShopLogoRating($Shop, $aNavElements);
    my ($HTMLHeadRating, $EvilNavElementID) = @{_getShopHTMLHeadRating($Shop, $aNavElements)};
    my $WebmasterToolsRating = _getWebmasterToolsRating($Shop->get('AdditionalHtmlHead') . $Shop->get('AdditionalHtmlHeadMobileSF'));
    my $SitemapRating = _getSitemapRating($hShopAttributes);

    my $hValues = {
        'DomainName' => {
            'Rating' => $DomainNameRating,
            'RatingShades' => [qw ( green yellow red )],
            'Percentage' => 0.15,
        },
        'HTMLHead' => {
            'Rating' => $HTMLHeadRating,
            'RatingShades' => [qw ( green red red )],
            'Percentage' => 0.15,
        },
        'WebmasterTools' => {
            'Rating' => $WebmasterToolsRating,
            'RatingShades' => [qw ( green red )],
            'Percentage' => 0.05,
        },
        'PageCache' => {
            'Rating' => $PageCacheRating,
            'RatingShades' => [qw ( green red )],
            'Percentage' => 0.10,
        },
        'CopyrightTrick' => {
            'Rating' => $CopyrightTrickRating,
            'RatingShades' => [qw ( green red )],
            'Percentage' => 0.05,
        },
        'ShopLogo' => {
            'Rating' => $ShopLogoRating,
            'RatingShades' => [qw ( green yellow red )],
            'Percentage' => 0.05,
        },
        'Sitemap' => {
            'Rating' => $SitemapRating,
            'RatingShades' => [qw ( green yellow red )],
            'Percentage' => 0.10,
        },
    };
    $hValues->{'HTMLHead'}->{'EvilNavElementID'} = $EvilNavElementID if defined $EvilNavElementID;
    return $hValues;
}

#========================================================================================
# function     CalculateShopSEOQuality
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Rating = CalculateShopSEOQuality($hValues)
#----------------------------------------------------------------------------------------
# description  Calculates a rating factor which determindes the SEO quality for the shop.
#----------------------------------------------------------------------------------------
# input        $hValues | values and ratings for SEO check | ref.hash
# return       $Rating | SEO quality from 0 .. 100 | integer
#========================================================================================
sub CalculateShopSEOQuality {
    my ($hValues) = @_;

    # Overall rating for the shop
    my $ShopQuality = 0;
    my $PercentageSum = 0;

    my @RatingKeys = grep { ref($hValues->{$_}) eq "HASH" and defined $hValues->{$_}->{'Rating'} and defined $hValues->{$_}->{'Percentage'} } keys %$hValues;

    my $Sum = 0;
    foreach my $Key (@RatingKeys) {
        $Sum += $hValues->{$Key}->{'Percentage'};
    }
    my $PercentageFactor = 1.0 / $Sum;

    foreach my $Key (@RatingKeys) {
        my $RatingClass = $hValues->{$Key}->{'RatingShades'}[$hValues->{$Key}->{'Rating'}-1];
        $hValues->{$Key}->{'RatingClass'} = $RatingClass;
        $hValues->{$Key}->{'RatingSign'} = GetRatingSign($RatingClass);

        my $NumShades = scalar(@{$hValues->{$Key}->{'RatingShades'}});

        my $Rating = $hValues->{$Key}->{'Rating'};
        my $Value;
        ## no critic qw(ProhibitMagicNumbers)
        if($NumShades == 2) {
            #(100% bei grn, 0% rot)
            $Value = $Rating == 1 ? 1 : 0;     # 0 || 1
        } elsif($NumShades == 3) {
            ##(100% bei grn, 50% gelb, 0% rot)
            if($Rating == 3) { $Value = 0; }
            elsif($Rating == 2) { $Value = 0.5; }
            else { $Value = 1; }
        } else {
            #(100% bei grn, 75% gelb, 50% orange, 0% rot)
            if($Rating == 4) { $Value = 0; }
            elsif($Rating == 3) { $Value = 0.75; }
            elsif($Rating == 2) { $Value = 0.5; }
            else { $Value = 1; }
        }

        ## use critic
        my $Percentage = $PercentageFactor * $hValues->{$Key}->{'Percentage'};
        $ShopQuality += $Percentage * $Value;
        $PercentageSum += $Percentage;
    }

    # Weighting should be 1 in sum
    # rounding issue workaround
    if(sprintf('%.0f', $PercentageSum*100) != 100) {    ## no critic qw(ProhibitMagicNumbers)
        Warning(
            'Weighting of shop SEO quality factors should be 100% in sum',
            {
                'PercentageSum' => $PercentageSum,
                'Values' => $hValues,
            }
        );
    }

    return sprintf('%.0f', $ShopQuality * 100);  ## no critic qw(ProhibitMagicNumbers)
}

#========================================================================================
# function     GetRatingSign
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Sign = GetRatingSign($RatingClass)
#----------------------------------------------------------------------------------------
# description  Returns the html character code for the given SEO rating class string.
#----------------------------------------------------------------------------------------
# input        $RatingClass | class string like "red", "yellow", ... | string
# return       $Sign | html character code | string
#========================================================================================
sub GetRatingSign {
    my ($RatingClass) = @_;

    if($RatingClass eq 'green') {
        return '&#10004;';
    } elsif($RatingClass eq 'yellow' or $RatingClass eq 'orange') {
        return '&#33;';
    }

    return '&#10008;';
}

#========================================================================================
# function     UpdateSEOQuality
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateSEOQuality($Object, $hValues);
#----------------------------------------------------------------------------------------
# example      my $Object = $Servlet->object;
#               my $hValues = PerformSEOCheck($Object);
#               UpdateSEOQuality($Object, $hValues);
#----------------------------------------------------------------------------------------
# description  Saves the new SEO quality values. See PerformSEOCheck method for details
#               of the $hValues parameter.
#               This method only traverses the per-language data of $hValues and uses
#               the contained 'NewSEOQuality' value to save those quality values.
#----------------------------------------------------------------------------------------
# input        $Object | content object | object
# input        $hValues | values and ratings for SEO check of the shop
#               <ul>
#                   <li>
#                       <ul>1
#                           <li>NewSEOQuality - SEO quality for the object for language id 1 - integer</li>
#                       </ul>
#                       <ul>2
#                           <li>NewSEOQuality - SEO quality for the object for language id 2 - integer</li>
#                       </ul>
#                       ...
#                   </li>
#                   <li>... </li>
#               </ul>
#               | ref.hash
#========================================================================================
sub UpdateSEOQuality {
    my ($Object, $hValues) = @_;

    # localized attributes
    my @ShopLanguageIDs = @{$Object->getSite()->languages};
    foreach my $LanguageID (@ShopLanguageIDs) {
        next unless exists $hValues->{$LanguageID}->{'NewSEOQuality'};
        $Object->set( { 'SEOQuality' => $hValues->{$LanguageID}->{'NewSEOQuality'}}, $LanguageID);
    }
}

#========================================================================================
# function     PerformSEOCheckAndUpdateObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       PerformSEOCheckAndUpdateObject($ContentObject)
#----------------------------------------------------------------------------------------
# description  Performs the calculation of the SEO quality. Then calls the UpdateSEOQuality
#               method to update the object.
#----------------------------------------------------------------------------------------
# input        $ContentObject | content object | object
#========================================================================================
sub PerformSEOCheckAndUpdateObject {
    my ($ContentObject) = @_;
    return unless $ContentObject->get('CanDoSEOCheck');

    my $hSEOValues = $ContentObject->get('CalculatedSEOCheckValues');
    UpdateSEOQuality($ContentObject, $hSEOValues);
}

#========================================================================================
# function     PerformSEOCheck
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = PerformSEOCheck($Object)
#----------------------------------------------------------------------------------------
# description  Performs the calculation of the SEO quality for all languages and
#               total SEO quality (SEO Average).
#----------------------------------------------------------------------------------------
# input        $Object | content object | object
# return       $hValues | values and ratings for SEO check of the shop
#               <ul>
#                   <li>
#                       <ul>1
#                           <li>NewSEOQuality - SEO quality for the object for language id 1 - integer</li>
#                           <li>NewSEOQualityClass - CSS class name depending on rating - string</li>
#                       </ul>
#                       ...
#                   </li>
#                   <li>NewSEOQualityAverage - total SEO quality for the object - integer</li>
#                   <li>NewSEOQualityClass - CSS class name depending on rating - string</li>
#                   <li>... </li>
#               </ul>
#               | ref.hash
#========================================================================================
sub PerformSEOCheck {
    my ($Object) = @_;

    my $Site = $Object->getSite;
    my $ShopDomain = $Site->get('DomainName');
    my $ShopWebUrl = $Site->get('WebUrl');

    my $hValues;

    # general attributes
    my $hGeneralValues = {
        FormerSEOQuality => 0,
        SiteUrl => $ShopWebUrl,
        Domain => $ShopDomain,
    };

    TriggerHook('SEOCheckTests', {
        'Object'        => $Object,
        'Shop'          => $Site,
        'Values'        => $hGeneralValues,
    });

    # localized attributes
    my @ShopLanguageIDs = @{$Object->getSite()->languages};
    my $TotalSEORating = 0;
    foreach my $LanguageID (@ShopLanguageIDs) {
        my $hLanguageCheckValues = GetSEOCheckValues($Site, $Object, $LanguageID);

        $hValues->{$LanguageID} = {
            %$hGeneralValues,
            %$hLanguageCheckValues,
        };

        my $LanguageCheckQuality = CalculateSEOQuality($hValues->{$LanguageID}, $Object);
        $hValues->{$LanguageID}->{'NewSEOQuality'} = $LanguageCheckQuality;
        $hValues->{$LanguageID}->{'NewSEOQualityClass'} = GetSEOQualityClass($LanguageCheckQuality);

        $TotalSEORating += $LanguageCheckQuality;
    }
    $TotalSEORating = sprintf('%.0f', $TotalSEORating / scalar(@ShopLanguageIDs));
    $hValues->{'NewSEOQualityAverage'} = $TotalSEORating;
    $hValues->{'NewSEOQualityClass'} = GetSEOQualityClass($hValues->{'NewSEOQuality'});

    return $hValues;
}



sub _getShopLogoRating {
    my ($Shop, $aNavElements) = @_;

    # has logo?
    return 3 unless $Shop->get('Logo');

    # has logo nav element?
    my $LogoID = LoadObjectByPath('/NavElements/Logo')->id;
    foreach my $NavElement (@$aNavElements) {
        if($NavElement->class->id == $LogoID) {
            my $URL = $NavElement->get('URL');
            # link has internal url?
            if($URL && $URL ne '' && _isInternalUrl($Shop, $URL)) {
                return 1;
            } else {
                return 2;
            }
        }
    }

    return 3;
}

sub _isInternalUrl {
    my ($Shop, $URL) = @_;

    return 0 if !$URL || $URL eq '';

    my $ProtocolAndServer = $Shop->get('ProtocolAndServer');
    my $ShopWebUrl = $Shop->get('WebUrl');

    return 1 if $URL =~ /^\Q$ProtocolAndServer\E/i;
    return 1 if $URL =~ /^\Q$ShopWebUrl\E/i;
    return 1 if $URL =~ /^?(ObjectPath|ObjectID|ViewAction)=/;  # relative
}

sub _getShopDomainNameRating {
    my ($ShopDomain) = @_;
    return 3 if !$ShopDomain || $ShopDomain eq '';

    # bad domain name
    return 2 if $ShopDomain =~ /\.eu$/i;
    return 2 unless EncodeDomain($ShopDomain) eq $ShopDomain;

    return 1;
}

sub _getShopCopyrightTrickRating {
    my ($Shop, $aNavElements) = @_;

    # look for TitleRepeat nav element in footer/bottom navbars
    my $NavBarFooterID = LoadObjectByPath('/NavBars/Footer')->id;
    my $NavBarBottomID = LoadObjectByPath('/NavBars/Bottom')->id;
    my $TitleRepeatID = LoadObjectByPath('/NavElements/TitleRepeat')->id;
    foreach my $NavElement (@$aNavElements) {
        my $NavBarID = $NavElement->get('NavBar')->id;
        if(($NavBarID == $NavBarFooterID || $NavBarID == $NavBarBottomID) && $NavElement->class->id == $TitleRepeatID) {
           return 1;
        }
    }

    return 2;
}


sub _getUrlRating {
    my ($ShopDomain, $Canonical, $ShopWebUrl) = @_;

    return 5 if !$ShopDomain || $ShopDomain eq '';
    return 4 if $Canonical =~ /\Q$ShopWebUrl\E/;

    my ($UrlLength) = $Canonical =~ /https?:\/\/(.*)/;
    $UrlLength = length($UrlLength);

    return 3 if $UrlLength > 200;   ## no critic qw(ProhibitMagicNumbers)
    return 2 if $UrlLength > 100;   ## no critic qw(ProhibitMagicNumbers)

    return 1;
}

sub _getTitleRating {
    my ($Name, $Title, $ShopTitle, $ShopName) = @_;

    return 4 if !$Title || $Title eq '';

    my $StrippedTitle = $Title;
    $StrippedTitle =~ s/[^[:alnum:]]//gi;
    return 4 if !$StrippedTitle || $StrippedTitle eq ''; # no actual words contained

    my $StrippedName = $Name;
    $StrippedName =~ s/[^[:alnum:]]//gi if $StrippedName;
    return 4 if !$StrippedName || $StrippedName eq ''; # no actual words contained

    # similarity check, at least 70% should match
    return 4 if CompareStrings($Name, $Title) < 0.7; ## no critic qw(ProhibitMagicNumbers)

    if($ShopTitle) {
        $ShopTitle =~ s/[^[:alnum:]]//gi;
        return 4 if $StrippedTitle eq $ShopTitle;
    }

    return 3 if length($Title) > 65;    ## no critic qw(ProhibitMagicNumbers)
    return 2 if $StrippedName && length($StrippedTitle) == length($StrippedName);

    return 1;
}

sub _getDescriptionRating {
    my ($Description) = @_;

    return 5 if !$Description || $Description eq '';
    ## no critic qw(ProhibitMagicNumbers)
    return 4 if length($Description) > 255;
    return 3 if length($Description) > 140;
    return 2 if length($Description) < 90;
    ## use critic

    return 1;
}

sub _getAdvDescriptionRating {
    my ($AdvDescription, $Description) = @_;
    return 5 if !$AdvDescription || $AdvDescription eq '';

    if($Description) {
        my $StrippedAdvDescription = $AdvDescription;
        $StrippedAdvDescription =~ s/[\s\W]//gi;

        my $StrippedDescription = $Description;
        $StrippedDescription =~ s/[\s\W]//gi;

        return 4 if lc($StrippedAdvDescription) eq lc($StrippedDescription);
    }

    while($AdvDescription =~ /(<img\s.*?>)/isgx) {
        my ($Match) = $1 =~ /(?:alt|title)="(.*?)"/ix;
        return 3 unless defined $Match and $Match =~ /\S/;
    }

    return 2 if $AdvDescription !~ /<(?:strong|b|i|em)>/isg &&
                $AdvDescription !~ /<(?:ul|ol)>/isg;

    return 1;
}

sub _getDepthRating {
    my ($Depth) = @_;

    return 4 if $Depth  > 6;
    return 3 if $Depth == 6;
    return 2 if $Depth == 5;
    return 1;
}

# returns array [ ratingNumber, navelementid ]
sub _getShopHTMLHeadRating {
    my ($Shop, $aNavElements) = @_;

    # look for eval code in custom html head
    return [2, undef] if !CheckForBadSEOHTML($Shop->get('AdditionalHtmlHead') . $Shop->get('AdditionalHtmlHeadMobileSF'));

    # look for eval code in customizable navelements
    my $NavElementBoxHTMLID = LoadObjectByPath('/NavElements/BoxElement')->id;
    my $NavElementHTMLID = LoadObjectByPath('/NavElements/TextElement')->id;
    my @ShopLanguageIDs = @{$Shop->languages};

    foreach my $NavElement (@$aNavElements) {
        next unless $NavElement->class->id == $NavElementBoxHTMLID || $NavElement->class->id == $NavElementHTMLID;

        foreach my $LanguageID (@ShopLanguageIDs) {
            return [3, $NavElement->id ] if !CheckForBadSEOHTML($NavElement->get('Text', $LanguageID));
        }
    }

    return [1, undef];
}

# checks for google webmaster tools verification meta tag
sub _getWebmasterToolsRating {
    my ($AdditionalHtmlHead) = @_;
    return 1 if $AdditionalHtmlHead and $AdditionalHtmlHead =~ /<meta\s.*?name="google-site-verification".*?>/sgx;
    return 0;
}

sub _getSitemapRating {
    my ($hShopAttributes) = @_;

    return 3 if $hShopAttributes->{ClosedByMerchant};
    return 2 unless $hShopAttributes->{SubmittedSitemapDomain};
    return 1;
}

#========================================================================================
# function     CheckForBadSEOHTML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsOk = CheckForBadSEOHTML($Code)
#----------------------------------------------------------------------------------------
# description  Checks if $Code contains any HTML markup that is horrible for SEO. This
#               for example are title and meta tags, entire new html documents, etc.
#----------------------------------------------------------------------------------------
# input        $Code | html code | string
# return       $IsOk | 1 of html code is ok, 0 if not | boolean
#========================================================================================
sub CheckForBadSEOHTML {
    my ($Code) = @_;
    return 1 if not defined $Code or $Code eq '';   # empty is good, nothing can break :-)

    return 0 if $Code =~ /<(?:[\/]?html|[\/]?head|[\/]?body|[\/]?title)>/isgx;
    return 0 if $Code =~ /<meta.*?name=["'](?:description|keywords|robots|googlebot|msnbot)["']/imx;     #'
    return 0 if $Code =~ /<link.*?rel=["']canonical["']/isgx;   #'

    return 1;
}

#========================================================================================
# function     PerformStartPageSEOCheck
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = PerformStartPageSEOCheck($Object)
#----------------------------------------------------------------------------------------
# description  Performs the calculation of the SEO quality for all languages and
#               total SEO quality (SEO Average) for the StartPage.
#----------------------------------------------------------------------------------------
# input        $Object | content object | object
# return       $hValues | values and ratings for SEO check of the shop
#               <ul>
#                   <li>
#                       <ul>1
#                           <li>NewSEOQuality - SEO quality for the object for language id 1 - integer</li>
#                           <li>NewSEOQualityClass - CSS class name depending on rating - string</li>
#                       </ul>
#                       ...
#                   </li>
#                   <li>NewSEOQualityAverage - total SEO quality for the object - integer</li>
#                   <li>NewSEOQualityClass - CSS class name depending on rating - string</li>
#                   <li>... </li>
#               </ul>
#               | ref.hash
#========================================================================================
sub PerformStartPageSEOCheck {
    my ($Object) = @_;

    my $Site = $Object->getSite;

    my $ShopDomain = $Site->get('DomainName');
    my $ShopWebUrl = $Site->get('WebUrl');

    my $hValues;

    # general attributes
    my $hGeneralValues = {
        FormerSEOQuality => $Object->get('SEOQuality') || 0,
        SiteUrl => $ShopWebUrl,
        Domain => $ShopDomain,
    };

    TriggerHook('SEOCheckTests', {
        'Object'        => $Object,
        'Shop'          => $Site,
        'Values'        => $hGeneralValues,
    });

    # localized attributes
    my @ShopLanguageIDs = @{$Object->getSite()->languages};
    my $TotalSEORating = 0;
    foreach my $LanguageID (@ShopLanguageIDs) {
        my $hLanguageCheckValues = GetSEOCheckValues($Site, $Object, $LanguageID);

        # adapt ratings for start page
        my $hFilteredValues = {
            'Description' => $hLanguageCheckValues->{'Description'},
            'AdvDescription' => $hLanguageCheckValues->{'AdvDescription'},
            'Title' => _GetStartPageTitleCheck($hLanguageCheckValues->{'Title'}->{'Value'}),
            'Url' => {
                'Value' => $hLanguageCheckValues->{'Url'}->{'Value'},
                'ValueWithoutHttp' => $hLanguageCheckValues->{'Url'}->{'ValueWithoutHttp'}
            }
        };

        $hValues->{$LanguageID} = {
            %$hGeneralValues,
            %$hFilteredValues,
        };

        my $LanguageCheckQuality = CalculateSEOQuality($hValues->{$LanguageID}, $Object);
        $hValues->{$LanguageID}->{'NewSEOQuality'} = $LanguageCheckQuality;
        $hValues->{$LanguageID}->{'NewSEOQualityClass'} = GetSEOQualityClass($LanguageCheckQuality);
        $TotalSEORating += $LanguageCheckQuality;
    }
    $TotalSEORating = sprintf('%.0f', $TotalSEORating / scalar(@ShopLanguageIDs));
    $hValues->{'NewSEOQualityAverage'} = $TotalSEORating;
    $hValues->{'NewSEOQualityClass'} = GetSEOQualityClass($TotalSEORating);
    return $hValues;
}

# returns hash of the seo check for the title of the start page
sub _GetStartPageTitleCheck {
    my ($Title) = @_;

    my $TitleRating;
    if(!$Title || $Title eq '') {
        $TitleRating = 5;
    } else {
        my $StrippedTitle = $Title;
        $StrippedTitle =~ s/[\s\W]//gi;

        ## no critic qw(ProhibitMagicNumbers)
        if(!$StrippedTitle) {
            $TitleRating = 5;
        } elsif(length($Title) > 65) {
            $TitleRating = 3;
        } else {
            $TitleRating = 1;
        }
        ## use critic
    }

    return {
        'Value' => $Title,
        'Rating' => $TitleRating,
        'RatingClass' => qw ( green yellow red red red )[$TitleRating-1],
        'Characters' => length($Title),
        'Words' => GetWordCount($Title)
    };
}



1;
