#========================================================================================
# package      DE_EPAGES::TaxUS::Hooks::Basket
# state        public
# description  Hook functions for basket to order process.
#========================================================================================
package DE_EPAGES::TaxUS::Hooks::Basket;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::TaxUSSystemDB::API::Zip2TaxUS qw(GetInfoZip2TaxUSByZipCodeCityState);    ## no critic qw(RequireCorrectImports)
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::Core::API::PerlTools qw(isValid);

use constant US_COUNTRYID => 840;

#========================================================================================
# function     OnAddressValidation
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAddressValidation( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_AddressValidation triggered, to have the possiblity to
#               check billing, shipping address.
#               Check US addresses by using zip2tax database
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnAddressValidation {
    my ($hParams) = @_;
    my $Servlet   = $hParams->{'Servlet'};
    my $Form      = $Servlet->form;

    #called on registered customer: NewShippingAddress, SaveBillingAddress, SaveShippingAndBillingAddress
    # and SF Customer account

    # test if we can switch this shop to TaxUS
    my $Shop = $Servlet->object->getSite();
    return unless $Shop->get('CanUseTaxUS');


    return unless $Form->value('CountryID') eq US_COUNTRYID;

    my $AddressType = 'SaveRegistration';
    $AddressType = 'BillingAddress'  if $Servlet->params('ChangeAction') eq 'SaveShippingAndBillingAddress';
    $AddressType = 'BillingAddress'  if $Servlet->params('ChangeAction') eq 'SaveBillingAddress';
    $AddressType = 'ShippingAddress' if $Servlet->params('ChangeAction') eq 'SaveShippingAddress';
    $AddressType = 'ShippingAddress' if $Servlet->params('ChangeAction') eq 'NewShippingAddress';

    my $hAddressParams = {};
    $hAddressParams->{'ZipCode'} = $Form->value('Zipcode');
    $hAddressParams->{'City'}    = $Form->value('City');
    $hAddressParams->{'State'}   = $Form->value('State');

    if (not isValid($hAddressParams->{'State'})) {
        $Form->addFormError({
            Reason => 'STATE_NOT_VALID',
            Name   => 'State',
            Form   => $AddressType,
        });
        $Form->executeFormError;
    }

    my $hInfo = GetInfoZip2TaxUSByZipCodeCityState($hAddressParams);

    # add form errors
    if (not exists $hInfo->{'ZipCode'}) {
        foreach my $Element (qw(Zipcode City State)) {
            $Form->addFormError({
                    Reason => 'INVALID_US_ADDRESS',
                    Name   => $Element,
                    Form   => $AddressType,
                    Value  => '',
            });
        }
        $Servlet->vars('EnableShippingAddress', 1) if $Servlet->params('ChangeAction') eq 'NewShippingAddress';
    }

    return;
}

#========================================================================================
# function     OnBasketAddressValidation
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasketAddressValidation( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_BasketAddressValidation triggered, to have the possiblity to
#               check billing, shipping address.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasketAddressValidation {
    my ($hParams) = @_;

    # called if a new customer with an new login is registered (SaveCustomerData)
    # old and new checkout
    return OnAddressValidation($hParams);
}

#========================================================================================
# function     OnAfterSaveMultiCheckoutAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterSaveMultiCheckoutAddress( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_AfterSaveMultiCheckoutAddress triggered, to interrupt order process to have the
#               possiblity to check billing, shipping address.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterSaveMultiCheckoutAddress {
    my ($hParams) = @_;
    my $Servlet   = $hParams->{'Servlet'};
    my $Basket    = $Servlet->object;

    # control selection of login type (normal login, address login with or without registration)
    my $CustomerType = $Servlet->params('CustomerType');

    # do not execute this action if we only login the customer (done by ChangeAction SaveLoginForm )
    return if defined $CustomerType and $CustomerType eq 'Member';

    my $Shop = $Basket->getSite();

    # test if we can switch this shop to TaxUS
    return unless $Shop->get('CanUseTaxUS');

    # test US addresses
    my $Form      = $Servlet->form;
    my $Container = $Basket->container;

    my @AddressTypes = ('BillingAddress');
    push @AddressTypes, 'ShippingAddress' if $Container->get('Shipping');
    my $ExistsError;

    foreach my $AddressType (@AddressTypes) {
        my $Address = $Basket->get($AddressType);
        next unless defined $Address;
        next unless $Address->get('CountryID') eq US_COUNTRYID;

        my $hAddressParams = $Address->get(['Zipcode', 'City', 'State']);
        $hAddressParams->{'ZipCode'} = $hAddressParams->{'Zipcode'};

        my $Prefix = '';
        $Prefix = 'Shipping'
        if $AddressType eq 'ShippingAddress'
            and (  ($CustomerType eq 'NonMember')
                or ($CustomerType eq 'GetMember')
                or ($Shop->get('OrderProcess') == 0));

        my $hInfo = GetInfoZip2TaxUSByZipCodeCityState($hAddressParams);

        # add form errors
        if (not exists $hInfo->{'ZipCode'}) {
            $ExistsError = 1;
            foreach my $Element (qw(Zipcode City State)) {
                $Form->addFormError({
                        Reason => 'INVALID_US_ADDRESS',
                        Name   => $Prefix . $Element,
                        Form   => $AddressType,
                        Value  => '',
                });
            }
            $Servlet->vars('IsMyBad', 1);

        }
    }

    # show basket template if something is not correct
    my $ViewAction = $Shop->get('OrderProcess') == 0 ? 'ViewAddress' : 'ViewMultiCheckoutAddress';

    Error(
        'INVALID_FORM', {
            'ViewVars'   => {},
            'ViewAction' => $ViewAction,
        }) if $ExistsError;
}

#========================================================================================
# function     OnBasket2OrderAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderAddress( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2Order triggered, to interrupt order process to have the
#               possiblity to check billing, shipping address. Its possible to
#               prevent the interruption with the basket attribute 'IsAddressOK' = 1.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Basket
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderAddress {
    my ($hParams) = @_;
    my $Basket = $hParams->{'Basket'};
    return if $Basket->get('IsAddressOK');

    # test if we can switch this shop to TaxUS
    my $Shop = $Basket->getSite();
    return unless $Shop->get('CanUseTaxUS');


    # old checkout and non registered customer
    # order process is interupted on saving address data
    return OnAfterSaveMultiCheckoutAddress($hParams);
}

1;
