#========================================================================================
# package      DE_EPAGES::UPS::API::DataCollectors
# state        public
#----------------------------------------------------------------------------------------
# description  collects data from order
#----------------------------------------------------------------------------------------
# example      my $obj = DE_EPAGES::UPS::API::DataCollectors->new($ShippingLabelUPS);
#               $obj->GetShippingFulFillmentMethodData();
#========================================================================================
package DE_EPAGES::UPS::API::DataCollectors;
use strict;
use Data::Dumper;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );

sub new {
    my $class = shift;
    my ($ShippingLabelUPS)  = @_;
    TestObject('ShippingLabelUPS',$ShippingLabelUPS,'DE_EPAGES::UPS::API::Object::ShippingLabelUPS');
    my $ShippingFulfillmentMethod = $ShippingLabelUPS->get('ShippingFulfillmentMethod');
    TestObject('ShippingFulfillmentMethod',$ShippingFulfillmentMethod,'DE_EPAGES::UPS::API::Object::ShippingFulfillmentMethodUPS');
    my $Order = $ShippingLabelUPS->get('Order');
    TestObject('Order',$Order,'DE_EPAGES::Order::API::Object::CustomerOrder');
    my $self  = bless {
                    'ShippingLabelUPS'          => $ShippingLabelUPS,
                    'Order'                     =>$Order,
                    'ShippingFulfillmentMethod' => $ShippingFulfillmentMethod,
                }, $class;
    return $self;
}

#========================================================================================
# function     getShippingFulFillmentMethodData
# state        public
#----------------------------------------------------------------------------------------
# syntax       getShippingFulFillmentMethodData();
#----------------------------------------------------------------------------------------
# description  returns all relevant Data from Order Object
#----------------------------------------------------------------------------------------
# return       $hShippingFulfillmentMethodData | shop related data |hash_ref
#========================================================================================
sub getShippingFulFillmentMethodData {
    my $self = shift;
    my $ShippingFulfillmentMethod = $self->{'ShippingFulfillmentMethod'};
    my @Values = qw (
                AccessLicenseNumber
                WebservicePassword
                IsInTestMode
                AutoGenerateLabels
                ShipperNumber
                WebserviceUsername
                Alias
    );
    my $hReturn = $self->_objectToHashByList($ShippingFulfillmentMethod,\@Values);
    return $hReturn;
}
#========================================================================================
# function     getShippingTypeData
# state        private
#----------------------------------------------------------------------------------------
# syntax       getShippingTypeData();
#----------------------------------------------------------------------------------------
# description  returns all relevant Data from Order Object
#----------------------------------------------------------------------------------------
# return       $hShippingTypeData | shipping type related data  |hash_ref
#========================================================================================
sub getShippingTypeData {
    my $self = shift;
    my $ShippingFulfillmentMethod = $self->{'ShippingFulfillmentMethod'};
    my @Values = qw (
                ShippingServiceURL
                ShippingServiceTestURL
                VoidServiceURL
                VoidServiceTestURL
                WeightUnit
                SizeUnit
    );
    return $self->_objectToHashByList($ShippingFulfillmentMethod->get('ShippingFulfillmentType'),\@Values);
}

#========================================================================================
# function     getAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       getAddress();
#----------------------------------------------------------------------------------------
# input        $AddressType | type like ShopAddress or Address(contains automaticly the
#               Billing Address or an Extra ShippingAdress) | String
#----------------------------------------------------------------------------------------
# description  returns all relevant Data from Order Object
#----------------------------------------------------------------------------------------
# return       $hShippingAddress | shipping Address related data  |hash_ref
#========================================================================================
sub getAddress {
    my $self = shift;
    my ($AddressType) = @_;
    my $Address = $self->{'ShippingLabelUPS'}->get($AddressType);
    my @Values = qw (
            Salutation
            DisplayName
            FullName
            Street
            Zipcode
            State
            City
            PhoneBusiness
            PhonePrivate
            PhoneCell
            Phone
            VATID
    );
    Error( 'MissingAddressRecord', { 'Parameter' => $AddressType } ) unless defined $Address;
    my $hReturn = $self->_objectToHashByList($Address,\@Values);
    $hReturn->{'CountryCode'} = $self->_getCountryCode($Address);
    $hReturn->{'Type'} = $AddressType;
    return $hReturn
}

#========================================================================================
# function     getShippingLabelData
# state        public
#----------------------------------------------------------------------------------------
# syntax       getShippingLabelData();
#----------------------------------------------------------------------------------------
# description  returns all relevant Data from ShippingLabel Object
#----------------------------------------------------------------------------------------
# return       $hShippingLabel | shipping label reated data | hash_ref
#========================================================================================
sub getShippingLabelData {
    my $self = shift;
    my $ShippingLabelUPS = $self->{'ShippingLabelUPS'};
    my @Values = qw (
            PackageDescription
            Amount
            SizeUnit
            WeightUnit
            TotalWeight
            ParcelWidth
            ParcelLength
            ParcelDepth
            SaturdayDelivery
            UPSCarbonNeutral
            AdultConfirmation
            DeliveryConfirmation
    );
     my $hReturn = $self->_objectToHashByList($ShippingLabelUPS,\@Values);
     $hReturn->{'ServiceCode'} = $ShippingLabelUPS->get('ServiceType')->get('ProductCode');
     $hReturn->{'PackagingCode'} = $ShippingLabelUPS->get('PackagingType')->get('ProductCode');
     return $hReturn;
}
#========================================================================================
# function     _objectToHashByList
# state        private
#----------------------------------------------------------------------------------------
# syntax       _objectToHashByList( $Object,$aDatafieldNames );
#----------------------------------------------------------------------------------------
# input        $Object | some ePagesObject | object
# input        $aDatafieldNames | a list of value names | refarray
#----------------------------------------------------------------------------------------
# description  copies all values which are stored in $aDatafieldNames from a incoming
#               Object into a hash.
#----------------------------------------------------------------------------------------
# return       $hReturnValues | a Hash with Values | refhash
#========================================================================================
sub _objectToHashByList{
    my $self = shift;
    my ($Object,$aDatafieldNames) = @_;
    my $hReturnValues;
    foreach my $DatafieldName (@$aDatafieldNames){
            $hReturnValues->{$DatafieldName} = $Object->get($DatafieldName)
    }
    return $hReturnValues;
}

#========================================================================================
# function     _getCountryCode
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getCountryCode( $ShippingAddress );
#----------------------------------------------------------------------------------------
# input        $ShippingAddress | object of ShippingAddress | object_ref
#----------------------------------------------------------------------------------------
# description  returns the Code2 e.g. "DE" or undef if not seted
#----------------------------------------------------------------------------------------
# return       $Code2 | Code2 e.g. "DE" or undef if not seted | String
#========================================================================================
sub _getCountryCode {
    my $self = shift;
    my ($ShippingAddress) = @_;
    my $Country = $ShippingAddress->get('Country');
    return undef unless(isValid($Country));
    return $Country->{'Code2'};
}
1;
