use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UPS::API::DataCollectors;
use Readonly;
use Test::More;
use Data::Dumper;

Readonly my $ROOTPATH => '/TestStore';
## no critic (ProhibitLongLines)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );
            # tests
            eval {
                my $ShippingLabelUPS = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1002/ShippingLabels/1001'); ## no critic (ProhibitPathConcatenation ProhibitUselessNoCritic)
                isa_ok($ShippingLabelUPS, 'DE_EPAGES::UPS::API::Object::ShippingLabelUPS');
                my $DataCollectors = DE_EPAGES::UPS::API::DataCollectors->new($ShippingLabelUPS);
                isa_ok($ShippingLabelUPS->get('Order'), 'DE_EPAGES::Order::API::Object::CustomerOrder');
                Test_ObjectToHashByList($DataCollectors,$ShippingLabelUPS->get('Order'));
                Test_GetShippingFulFillmentMethodData($DataCollectors);
                Test_GetShippingAddress($DataCollectors);
                Test_GetShopAddress($DataCollectors);
                Test_GetShippingTypeData($DataCollectors);
                Test_getShippingLabelData($DataCollectors);
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'importOrder.xml', $ROOTPATH );
    return 1;
}
sub Test_ObjectToHashByList {
    my ($DataCollectors,$Order) = @_;
    my $ShippingOrder = $Order->get('ShippingAddress');
    isa_ok($ShippingOrder, 'DE_EPAGES::Address::API::Object::Address');
    my @Values = qw ( Alias Street Title );
    my $hOneValue = $DataCollectors->_objectToHashByList($Order->get('ShippingAddress'),\@Values);## no critic (ProhibitPrivateSubroutineUse)
    is($hOneValue->{'Alias'},'ShippingAddress','fetch Alias');
    is($hOneValue->{'Street'},'Blumenweg 2','fetch Street');
    is($hOneValue->{'Title'},undef,'fetch Title');
}
sub Test_GetShippingAddress{
    my ($DataCollectors) = @_;
    my $hShippingAddress = $DataCollectors->getAddress('Address');
    is (scalar keys %$hShippingAddress,14,'Test_GetShippingAddress right size of return hash');## no critic (ProhibitMagicNumbers)
    is ($hShippingAddress->{'Type'},'Address','Address Type');
    is ($hShippingAddress->{'Street'},'Blumenweg 2','Street');
    is ($hShippingAddress->{'Salutation'},'Herr','Salutation');
    is ($hShippingAddress->{'DisplayName'},'Max Mustermann','DisplayName');
    is ($hShippingAddress->{'Zipcode'},'09876','Zipcode');
    is ($hShippingAddress->{'State'},'Deutschland','State');
    is ($hShippingAddress->{'City'},'Neustadt','City');
    is ($hShippingAddress->{'Phone'},'040123456789','Phone');
    is ($hShippingAddress->{'PhoneCell'},'040123456781','PhoneCell');
    is ($hShippingAddress->{'CountryCode'},'DE','CountryCode');
    is ($hShippingAddress->{'VATID'},undef,'VATID');
}

sub Test_GetShopAddress{
    my ($DataCollectors) = @_;
    my $hShopAddress = $DataCollectors->getAddress('ShopAddress');
    is (scalar keys %$hShopAddress,14,'Test_GetShopAddress right size of return hash');## no critic (ProhibitMagicNumbers)
    is ($hShopAddress->{'Type'},'ShopAddress','Address Type');
    is ($hShopAddress->{'Street'},'ShopStreet','Street');
    is ($hShopAddress->{'Salutation'},undef,'Salutation');
    is ($hShopAddress->{'DisplayName'},'ShopFirstName ShopLastName','DisplayName');
    is ($hShopAddress->{'Zipcode'},'123456','Zipcode');
    is ($hShopAddress->{'State'},undef,'State');
    is ($hShopAddress->{'City'},'ShopCity','City');
    is ($hShopAddress->{'Phone'},'123456','Phone');
    is ($hShopAddress->{'CountryCode'},'DE','CountryCode');
    is ($hShopAddress->{'VATID'},'123456789','VATID');
}
sub Test_GetShippingFulFillmentMethodData{
    my ($DataCollectors) = @_;
    my $hValues = $DataCollectors->getShippingFulFillmentMethodData();
    is (scalar keys %$hValues,7,'Test_GetShippingFulFillmentMethodData right size of return hash');
    is ($hValues->{'AccessLicenseNumber'},'AccessLicenseNumber','AccessLicenseNumber');
    is ($hValues->{'WebservicePassword'},'WebservicePassword','WebservicePassword');
    is ($hValues->{'IsInTestMode'},1,'IsInTestMode');
    is ($hValues->{'AutoGenerateLabels'},1,'AutoGenerateLabels');
    is ($hValues->{'ShipperNumber'},'ShipperNumber','ShipperNumber');
    is ($hValues->{'WebserviceUsername'},'WebserviceUsername','WebserviceUsername');
    is ($hValues->{'Alias'},'UPSAlias','Alias');
}
sub Test_GetShippingTypeData{
    my ($DataCollectors) = @_;
    my $hValues = $DataCollectors->getShippingTypeData();
    ok ($hValues->{'ShippingServiceURL'},'ShippingServiceURL');
    ok ($hValues->{'ShippingServiceTestURL'},'ShippingServiceTestURL');
    ok ($hValues->{'VoidServiceURL'},'VoidServiceURL');
    ok ($hValues->{'VoidServiceTestURL'},'VoidServiceTestURL');
    ok ($hValues->{'WeightUnit'},'WeightUnit');
    ok ($hValues->{'SizeUnit'},'SizeUnit');
}

sub Test_getShippingLabelData{
    my ($DataCollectors) = @_;
    my $hValues = $DataCollectors->getShippingLabelData();
    is ($hValues->{'PackageDescription'},'ShippingLabelUPS Description','Test_getShippingLabelData PackageDescription');
    is ($hValues->{'ServiceCode'},'11','Test_getShippingLabelData ServiceCode');
    is ($hValues->{'PackagingCode'},'02','Test_getShippingLabelData PackagingCode');
    is ($hValues->{'TotalWeight'},'1.1234','Test_getShippingLabelData TotalWeight');
    is ($hValues->{'ParcelWidth'},'1111','Test_getShippingLabelData ParcelWidth');
    is ($hValues->{'ParcelLength'},'2222','Test_getShippingLabelData ParcelLength');
    is ($hValues->{'ParcelDepth'},'3333','Test_getShippingLabelData ParcelDepth');
    is ($hValues->{'SaturdayDelivery'},'1','Test_getShippingLabelData SaturdayDelivery');
    is ($hValues->{'UPSCarbonNeutral'},'1','Test_getShippingLabelData UPSCarbonNeutral');
    is ($hValues->{'DeliveryConfirmation'},'1','Test_getShippingLabelData DeliveryConfirmation');
    is ($hValues->{'AdultConfirmation'},'1','Test_getShippingLabelData AdultConfirmation');
}
RunScript( Sub => \&Main );

1;
